@@ -24,6 +24,7 @@ import java.awt.AWTPermission;
 
 import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import net.sourceforge.jnlp.util.logging.OutputController;
 
 /**
  * The security element.
@@ -33,6 +34,75 @@ import net.sourceforge.jnlp.runtime.JNLPRuntime;
  */
 public class SecurityDesc {
 
+    /**
+     * Represents the security level requested by an applet/application, as specified in its JNLP or HTML.
+     */
+    public enum RequestedPermissionLevel {
+        NONE(null, null),
+        DEFAULT(null, "default"),
+        SANDBOX(null, "sandbox"),
+        J2EE("j2ee-application-client-permissions", null),
+        ALL("all-permissions", "all-permissions");
+
+        public static final String PERMISSIONS_NAME = "permissions";
+        private final String jnlpString, htmlString;
+
+        private RequestedPermissionLevel(final String jnlpString, final String htmlString) {
+            this.jnlpString = jnlpString;
+            this.htmlString = htmlString;
+        }
+
+        /**
+         * This permission level, as it would appear requested in a JNLP file. null if this level
+         * is NONE (unspecified) or cannot be requested in a JNLP file.
+         * @return the String level
+         */
+        public String toJnlpString() {
+            return this.jnlpString;
+        }
+
+        /**
+         * This permission level, as it would appear requested in an HTML file. null if this level
+         * is NONE (unspecified) or cannot be requested in an HTML file.
+         * @return the String level
+         */
+        public String toHtmlString() {
+            return this.htmlString;
+        }
+
+        /**
+         * The JNLP permission level corresponding to the given String. If null is given, null comes
+         * back. If there is no permission level that can be granted in JNLP matching the given String,
+         * null is also returned.
+         * @param jnlpString the JNLP permission String
+         * @return the matching RequestedPermissionLevel
+         */
+        public RequestedPermissionLevel fromJnlpString(final String jnlpString) {
+            for (final RequestedPermissionLevel level : RequestedPermissionLevel.values()) {
+                if (level.jnlpString != null && level.jnlpString.equals(jnlpString)) {
+                    return level;
+                }
+            }
+            return null;
+        }
+
+        /**
+         * The HTML permission level corresponding to the given String. If null is given, null comes
+         * back. If there is no permission level that can be granted in HTML matching the given String,
+         * null is also returned.
+         * @param jnlpString the JNLP permission String
+         * @return the matching RequestedPermissionLevel
+         */
+        public RequestedPermissionLevel fromHtmlString(final String htmlString) {
+            for (final RequestedPermissionLevel level : RequestedPermissionLevel.values()) {
+                if (level.htmlString != null && level.htmlString.equals(htmlString)) {
+                    return level;
+                }
+            }
+            return null;
+        }
+    }
+
     /*
      * We do not verify security here, the classloader deals with security
      */
@@ -46,6 +116,9 @@ public class SecurityDesc {
     /** J2EE permissions. */
     public static final Object J2EE_PERMISSIONS = "J2SE";
 
+    /** requested permissions type according to HTML or JNLP */
+    private final RequestedPermissionLevel requestedPermissionLevel;
+
     /** permissions type */
     private Object type;
 
@@ -87,6 +160,7 @@ public class SecurityDesc {
     private static Permission sandboxPermissions[] = {
             new SocketPermission("localhost:1024-", "listen"),
             // new SocketPermission("<DownloadHost>", "connect, accept"), // added by code
+            new PropertyPermission("java.util.Arrays.useLegacyMergeSort", "read,write"),
             new PropertyPermission("java.version", "read"),
             new PropertyPermission("java.vendor", "read"),
             new PropertyPermission("java.vendor.url", "read"),
@@ -145,11 +219,16 @@ public class SecurityDesc {
      * Create a security descriptor.
      *
      * @param file the JNLP file
+     * @param requestedPermissionLevel the permission level specified in the JNLP
      * @param type the type of security
      * @param downloadHost the download host (can always connect to)
      */
-    public SecurityDesc(JNLPFile file, Object type, String downloadHost) {
+    public SecurityDesc(JNLPFile file, RequestedPermissionLevel requestedPermissionLevel, Object type, String downloadHost) {
+        if (file == null) {
+            throw new NullJnlpFileException();
+        }
         this.file = file;
+        this.requestedPermissionLevel = requestedPermissionLevel;
         this.type = type;
         this.downloadHost = downloadHost;
 
@@ -159,6 +238,17 @@ public class SecurityDesc {
         customTrustedPolicy = getCustomTrustedPolicy();
     }
 
+    /**
+     * Create a security descriptor.
+     *
+     * @param file the JNLP file
+     * @param type the type of security
+     * @param downloadHost the download host (can always connect to)
+     */
+    public SecurityDesc(JNLPFile file, Object type, String downloadHost) {
+        this(file, RequestedPermissionLevel.NONE, type, downloadHost);
+    }
+
     /**
      * Returns a Policy object that represents a custom policy to use instead
      * of granting {@link AllPermission} to a {@link CodeSource}
@@ -177,7 +267,7 @@ public class SecurityDesc {
                 URI policyUri = new URI("file://" + policyLocation);
                 policy = Policy.getInstance("JavaPolicy", new URIParameter(policyUri));
             } catch (Exception e) {
-                e.printStackTrace();
+                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, e);
             }
         }
         // return the appropriate policy, or null
@@ -220,6 +310,13 @@ public class SecurityDesc {
         return permissions;
     }
 
+    /**
+     * @return the permission level requested in the JNLP
+     */
+    public RequestedPermissionLevel getRequestedPermissionLevel() {
+        return requestedPermissionLevel;
+    }
+
     /**
      * Returns a PermissionCollection containing the sandbox permissions
      */
@@ -233,10 +330,16 @@ public class SecurityDesc {
         if (grantAwtPermissions) {
             permissions.add(new AWTPermission("showWindowWithoutWarningBanner"));
         }
-
-        if (file.isApplication())
-            for (int i = 0; i < jnlpRIAPermissions.length; i++)
-                permissions.add(jnlpRIAPermissions[i]);
+        if (JNLPRuntime.isWebstartApplication()) {
+            if (file == null) {
+                throw new NullJnlpFileException("Can not return sandbox permissions, file is null");
+            }
+            if (file.isApplication()) {
+                for (int i = 0; i < jnlpRIAPermissions.length; i++) {
+                    permissions.add(jnlpRIAPermissions[i]);
+                }
+            }
+        }
 
         if (downloadHost != null && downloadHost.length() > 0)
             permissions.add(new SocketPermission(downloadHost,
