@@ -36,9 +36,12 @@ import java.util.Properties;
 import java.util.Set;
 
 import javax.naming.ConfigurationException;
+import javax.swing.JOptionPane;
 
+import net.sourceforge.jnlp.cache.CacheLRUWrapper;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.util.FileUtils;
+import net.sourceforge.jnlp.util.logging.OutputController;
 
 /**
  * Manages the various properties and configuration related to deployment.
@@ -48,8 +51,10 @@ import net.sourceforge.jnlp.util.FileUtils;
  */
 public final class DeploymentConfiguration {
 
-    public static final String DEPLOYMENT_DIR = ".icedtea";
-    public static final String DEPLOYMENT_CONFIG = "deployment.config";
+    public static final String DEPLOYMENT_SUBDIR_DIR = "icedtea-web";
+    public static final String DEPLOYMENT_CACHE_DIR = ".cache" + File.separator + DEPLOYMENT_SUBDIR_DIR;
+    public static final String DEPLOYMENT_CONFIG_DIR = ".config" + File.separator + DEPLOYMENT_SUBDIR_DIR;
+    public static final String DEPLOYMENT_CONFIG_FILE = "deployment.config";
     public static final String DEPLOYMENT_PROPERTIES = "deployment.properties";
     public static final String APPLET_TRUST_SETTINGS = ".appletTrustSettings";
 
@@ -60,14 +65,31 @@ public final class DeploymentConfiguration {
     public static final int JNLP_ASSOCIATION_ASK_USER = 2;
     public static final int JNLP_ASSOCIATION_REPLACE_ASK = 3;
 
-    /*
-     * FIXME these should be moved into JavaConsole, but there is a strange
-     * dependency in the build system. First all of netx is built. Then the
-     * plugin is built. So we cannot refer to plugin code in here :(
+    /**
+     * when set to as value of KEY_CONSOLE_STARTUP_MODE = "deployment.console.startup.mode",
+     * then console is not visible by default, but may be shown
      */
     public static final String CONSOLE_HIDE = "HIDE";
+    /**
+     * when set to as value of KEY_CONSOLE_STARTUP_MODE = "deployment.console.startup.mode",
+     * then console show for both javaws and plugin
+     */
     public static final String CONSOLE_SHOW = "SHOW";
+    /**
+     * when set to as value of KEY_CONSOLE_STARTUP_MODE = "deployment.console.startup.mode",
+     * then console is not visible by default, nop data are passed to it (save memory and cpu) but can not be shown
+     */
     public static final String CONSOLE_DISABLE = "DISABLE";
+    /**
+     * when set to as value of KEY_CONSOLE_STARTUP_MODE = "deployment.console.startup.mode",
+     * then console show for  plugin
+     */
+    public static final String CONSOLE_SHOW_PLUGIN = "SHOW_PLUGIN_ONLY";
+    /**
+     * when set to as value of KEY_CONSOLE_STARTUP_MODE = "deployment.console.startup.mode",
+     * then console show for javaws
+     */
+    public static final String CONSOLE_SHOW_JAVAWS = "SHOW_JAVAWS_ONLY";
 
     public static final String KEY_USER_CACHE_DIR = "deployment.user.cachedir";
     public static final String KEY_USER_PERSISTENCE_CACHE_DIR = "deployment.user.pcachedir";
@@ -123,8 +145,13 @@ public final class DeploymentConfiguration {
     /*
      * Networking
      */
+
+    /** the proxy type. possible values are {@code JNLPProxySelector.PROXY_TYPE_*} */
     public static final String KEY_PROXY_TYPE = "deployment.proxy.type";
+
+    /** Boolean. If true, the http host/port should be used for https and ftp as well */
     public static final String KEY_PROXY_SAME = "deployment.proxy.same";
+
     public static final String KEY_PROXY_AUTO_CONFIG_URL = "deployment.proxy.auto.config.url";
     public static final String KEY_PROXY_BYPASS_LIST = "deployment.proxy.bypass.list";
     public static final String KEY_PROXY_BYPASS_LOCAL = "deployment.proxy.bypass.local";
@@ -139,15 +166,27 @@ public final class DeploymentConfiguration {
     public static final String KEY_PROXY_OVERRIDE_HOSTS = "deployment.proxy.override.hosts";
 
     /*
-     * Tracing and Logging
+     * Logging
      */
-    public static final String KEY_ENABLE_TRACING = "deployment.trace";
-    public static final String KEY_ENABLE_LOGGING = "deployment.log";
+    public static final String KEY_ENABLE_LOGGING = "deployment.log"; //same as verbose or ICEDTEAPLUGIN_DEBUG=true
+    public static final String KEY_ENABLE_LOGGING_HEADERS = "deployment.log.headers"; //will add header OutputContorll.getHeader To all messages
+    public static final String KEY_ENABLE_LOGGING_TOFILE = "deployment.log.file";
+    public static final String KEY_ENABLE_LOGGING_TOSTREAMS = "deployment.log.stdstreams";
+    public static final String KEY_ENABLE_LOGGING_TOSYSTEMLOG = "deployment.log.system";
 
-    /*
-     * Console
+    /**
+     * Console initial status.
+     * One of CONSOLE_* values
+     * See declaration above:
+     * CONSOLE_HIDE = "HIDE";
+     * CONSOLE_SHOW = "SHOW";
+     * CONSOLE_DISABLE = "DISABLE";
+     * CONSOLE_SHOW_PLUGIN = "SHOW_PLUGIN_ONLY";
+     * CONSOLE_SHOW_JAVAWS = "SHOW_JAVAWS_ONLY";
      */
     public static final String KEY_CONSOLE_STARTUP_MODE = "deployment.console.startup.mode";
+    
+
 
     /*
      * Desktop Integration
@@ -167,6 +206,20 @@ public final class DeploymentConfiguration {
      */
     public static final String KEY_PLUGIN_JVM_ARGUMENTS= "deployment.plugin.jvm.arguments";
     public static final String KEY_JRE_DIR= "deployment.jre.dir";
+    private ConfigurationException loadingException = null;
+
+    public void setLoadingException(ConfigurationException ex) {
+        loadingException = ex;
+    }
+
+    public ConfigurationException getLoadingException() {
+        return loadingException;
+    }
+
+    public void resetToDefaults() {
+        currentConfiguration = Defaults.getDefaults();
+    }
+    
 
     public enum ConfigType {
         System, User
@@ -175,14 +228,13 @@ public final class DeploymentConfiguration {
     /** is it mandatory to load the system properties? */
     private boolean systemPropertiesMandatory = false;
 
-    /** The system's deployment.config file */
+    /** The system's subdirResult deployment.config file */
     private File systemPropertiesFile = null;
-    /** The user's deployment.config file */
+    /** The user's subdirResult deployment.config file */
     private File userPropertiesFile = null;
     
     /*default user file*/
-    public static final  File USER_DEPLOYMENT_PROPERTIES_FILE = new File(System.getProperty("user.home") + File.separator + DEPLOYMENT_DIR
-                + File.separator + DEPLOYMENT_PROPERTIES);
+    public static final File USER_DEPLOYMENT_PROPERTIES_FILE = new File(Defaults.USER_CONFIG_HOME + File.separator + DEPLOYMENT_PROPERTIES);
 
     /** the current deployment properties */
     private Map<String, Setting<String>> currentConfiguration;
@@ -199,15 +251,14 @@ public final class DeploymentConfiguration {
      * Initialize this deployment configuration by reading configuration files.
      * Generally, it will try to continue and ignore errors it finds (such as file not found).
      *
-     * @throws DeploymentException if it encounters a fatal error.
+     * @throws ConfigurationException if it encounters a fatal error.
      */
     public void load() throws ConfigurationException {
         load(true);
     }
 
     public static File getAppletTrustUserSettingsPath() {
-        return new File(System.getProperty("user.home") + File.separator + DEPLOYMENT_DIR
-                + File.separator + APPLET_TRUST_SETTINGS);
+        return new File(Defaults.USER_CONFIG_HOME + File.separator + APPLET_TRUST_SETTINGS);
     }
 
      public static File getAppletTrustGlobalSettingsPath() {
@@ -222,7 +273,7 @@ public final class DeploymentConfiguration {
      *
      * @param fixIssues If true, fix issues that are discovered when reading configuration by
      * resorting to the default values
-     * @throws DeploymentException if it encounters a fatal error.
+     * @throws ConfigurationException if it encounters a fatal error.
      */
     public void load(boolean fixIssues) throws ConfigurationException {
         // make sure no state leaks if security check fails later on
@@ -244,15 +295,13 @@ public final class DeploymentConfiguration {
         Map<String, Setting<String>> systemProperties = null;
 
         /*
-         * First, try to read the system's deployment.config file to find if
+         * First, try to read the system's subdirResult deployment.config file to find if
          * there is a system-level deployment.poperties file
          */
 
         if (systemConfigFile != null) {
             if (loadSystemConfiguration(systemConfigFile)) {
-                if (JNLPRuntime.isDebug()) {
-                    System.out.println("System level " + DEPLOYMENT_CONFIG + " is mandatory: " + systemPropertiesMandatory);
-                }
+                OutputController.getLogger().log("System level " + DEPLOYMENT_CONFIG_FILE + " is mandatory: " + systemPropertiesMandatory);
                 /* Second, read the System level deployment.properties file */
                 systemProperties = loadProperties(ConfigType.System, systemPropertiesFile,
                         systemPropertiesMandatory);
@@ -270,7 +319,7 @@ public final class DeploymentConfiguration {
         }
 
         /*
-         * Third, read the user's deployment.properties file
+         * Third, read the user's subdirResult deployment.properties file
          */
         userPropertiesFile = userFile;
         Map<String, Setting<String>> userProperties = loadProperties(ConfigType.User, userPropertiesFile, false);
@@ -390,9 +439,9 @@ public final class DeploymentConfiguration {
         for (String key : initial.keySet()) {
             Setting<String> s = initial.get(key);
             if (!(s.getName().equals(key))) {
-                System.out.println(R("DCInternal", "key " + key + " does not match setting name " + s.getName()));
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, R("DCInternal", "key " + key + " does not match setting name " + s.getName()));
             } else if (!defaults.containsKey(key)) {
-                System.out.println(R("DCUnknownSettingWithName", key));
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, R("DCUnknownSettingWithName", key));
             } else {
                 ValueValidator checker = defaults.get(key).getValidator();
                 if (checker == null) {
@@ -402,8 +451,9 @@ public final class DeploymentConfiguration {
                 try {
                     checker.validate(s.getValue());
                 } catch (IllegalArgumentException e) {
-                    System.out.println(R("DCIncorrectValue", key, s.getValue(), checker.getPossibleValues()));
+                    OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, R("DCIncorrectValue", key, s.getValue(), checker.getPossibleValues()));
                     s.setValue(s.getDefaultValue());
+                    OutputController.getLogger().log(e);
                 }
             }
         }
@@ -414,7 +464,7 @@ public final class DeploymentConfiguration {
      */
     private File findSystemConfigFile() {
         File etcFile = new File(File.separator + "etc" + File.separator + ".java" + File.separator
-                + "deployment" + File.separator + DEPLOYMENT_CONFIG);
+                + "deployment" + File.separator + DEPLOYMENT_CONFIG_FILE);
         if (etcFile.isFile()) {
             return etcFile;
         }
@@ -427,16 +477,16 @@ public final class DeploymentConfiguration {
                 jrePath = jreSetting.getValue();
             }
         } catch (Exception ex) {
-            ex.printStackTrace();
+            OutputController.getLogger().log(ex);
         }
 
         File jreFile;
         if (jrePath != null) {
             jreFile = new File(jrePath + File.separator + "lib"
-                    + File.separator + DEPLOYMENT_CONFIG);
+                    + File.separator + DEPLOYMENT_CONFIG_FILE);
         } else {
             jreFile = new File(System.getProperty("java.home") + File.separator + "lib"
-                    + File.separator + DEPLOYMENT_CONFIG);
+                    + File.separator + DEPLOYMENT_CONFIG_FILE);
         }
         if (jreFile.isFile()) {
             return jreFile;
@@ -449,19 +499,16 @@ public final class DeploymentConfiguration {
      * Reads the system configuration file and sets the relevant
      * system-properties related variables
      */
-    private boolean loadSystemConfiguration(File configFile) {
+    private boolean loadSystemConfiguration(File configFile) throws ConfigurationException {
 
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("Loading system configuation from: " + configFile);
-        }
+        OutputController.getLogger().log("Loading system configuation from: " + configFile);
 
         Map<String, Setting<String>> systemConfiguration = new HashMap<String, Setting<String>>();
         try {
             systemConfiguration = parsePropertiesFile(configFile);
         } catch (IOException e) {
-            if (JNLPRuntime.isDebug()) {
-                System.out.println("No System level " + DEPLOYMENT_PROPERTIES + " found.");
-            }
+            OutputController.getLogger().log("No System level " + DEPLOYMENT_CONFIG_FILE + " found.");
+            OutputController.getLogger().log(e);
             return false;
         }
 
@@ -469,36 +516,36 @@ public final class DeploymentConfiguration {
          * at this point, we have read the system deployment.config file
          * completely
          */
-
+        String urlString = null;
         try {
-            String urlString = systemConfiguration.get("deployment.system.config").getValue();
-            if (urlString == null) {
-                if (JNLPRuntime.isDebug()) {
-                    System.out.println("No System level " + DEPLOYMENT_PROPERTIES + " found.");
-                }
+            Setting<String> urlSettings = systemConfiguration.get("deployment.system.config");
+            if (urlSettings == null || urlSettings.getValue() == null) {
+                OutputController.getLogger().log("No System level " + DEPLOYMENT_PROPERTIES + " found in "+configFile.getAbsolutePath());
                 return false;
             }
+            urlString = urlSettings.getValue();
+            Setting<String> mandatory = systemConfiguration.get("deployment.system.config.mandatory");
+            systemPropertiesMandatory = Boolean.valueOf(mandatory == null ? null : mandatory.getValue()); //never null
+            OutputController.getLogger().log("System level settings " + DEPLOYMENT_PROPERTIES + " are mandatory:" + systemPropertiesMandatory);
             URL url = new URL(urlString);
             if (url.getProtocol().equals("file")) {
                 systemPropertiesFile = new File(url.getFile());
-                if (JNLPRuntime.isDebug()) {
-                    System.out.println("Using System level" + DEPLOYMENT_PROPERTIES + ": "
-                            + systemPropertiesFile);
-                }
-                Setting<String> mandatory = systemConfiguration.get("deployment.system.config.mandatory");
-                systemPropertiesMandatory = Boolean.valueOf(mandatory == null ? null : (String) mandatory.getValue());
+                OutputController.getLogger().log("Using System level" + DEPLOYMENT_PROPERTIES + ": " + systemPropertiesFile);
                 return true;
             } else {
-                if (JNLPRuntime.isDebug()) {
-                    System.out.println("Remote + " + DEPLOYMENT_PROPERTIES + " not supported");
-                }
+                OutputController.getLogger().log("Remote + " + DEPLOYMENT_PROPERTIES + " not supported: " + urlString + "in " + configFile.getAbsolutePath());
                 return false;
             }
         } catch (MalformedURLException e) {
-            if (JNLPRuntime.isDebug()) {
-                System.out.println("Invalid url for " + DEPLOYMENT_PROPERTIES);
+            OutputController.getLogger().log("Invalid url for " + DEPLOYMENT_PROPERTIES+ ": " + urlString + "in " + configFile.getAbsolutePath());
+            OutputController.getLogger().log(e);
+            if (systemPropertiesMandatory){
+                ConfigurationException ce = new ConfigurationException("Invalid url to system properties, which are mandatory");
+                ce.initCause(e);
+                throw ce;
+            } else {
+                return false;
             }
-            return false;
         }
     }
 
@@ -514,9 +561,7 @@ public final class DeploymentConfiguration {
     private Map<String, Setting<String>> loadProperties(ConfigType type, File file, boolean mandatory)
             throws ConfigurationException {
         if (file == null || !file.isFile()) {
-            if (JNLPRuntime.isDebug()) {
-                System.out.println("No " + type.toString() + " level " + DEPLOYMENT_PROPERTIES + " found.");
-            }
+            OutputController.getLogger().log("No " + type.toString() + " level " + DEPLOYMENT_PROPERTIES + " found.");
             if (!mandatory) {
                 return null;
             } else {
@@ -524,12 +569,16 @@ public final class DeploymentConfiguration {
             }
         }
 
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("Loading " + type.toString() + " level properties from: " + file);
-        }
+        OutputController.getLogger().log("Loading " + type.toString() + " level properties from: " + file);
         try {
             return parsePropertiesFile(file);
         } catch (IOException e) {
+            if (mandatory){
+                ConfigurationException ce = new ConfigurationException("Exception during loading of " + file + " which is mandatory to read");
+                ce.initCause(e);
+                throw ce;
+            }
+            OutputController.getLogger().log(e);
             return null;
         }
     }
@@ -550,15 +599,13 @@ public final class DeploymentConfiguration {
             sm.checkWrite(userPropertiesFile.toString());
         }
 
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("Saving properties into " + userPropertiesFile.toString());
-        }
+        OutputController.getLogger().log("Saving properties into " + userPropertiesFile.toString());
         Properties toSave = new Properties();
 
         for (String key : currentConfiguration.keySet()) {
             String oldValue = unchangeableConfiguration.get(key) == null ? null
-                    : (String) unchangeableConfiguration.get(key).getValue();
-            String newValue = currentConfiguration.get(key) == null ? null : (String) currentConfiguration
+                    : unchangeableConfiguration.get(key).getValue();
+            String newValue = currentConfiguration.get(key) == null ? null : currentConfiguration
                     .get(key).getValue();
             if (oldValue == null && newValue == null) {
                 continue;
@@ -593,7 +640,6 @@ public final class DeploymentConfiguration {
      * Reads a properties file and returns a map representing the properties
      *
      * @param propertiesFile the file to read Properties from
-     * @param destination the map to which all the properties should be added
      * @throws IOException if an IO problem occurs
      */
     private Map<String, Setting<String>> parsePropertiesFile(File propertiesFile) throws IOException {
@@ -668,7 +714,7 @@ public final class DeploymentConfiguration {
      */
     @SuppressWarnings("unused")
     private static void dumpConfiguration(Map<String, Setting<String>> config, PrintStream out) {
-        System.out.println("KEY: VALUE [Locked]");
+        OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "KEY: VALUE [Locked]");
 
         for (String key : config.keySet()) {
             Setting<String> value = config.get(key);
@@ -676,4 +722,138 @@ public final class DeploymentConfiguration {
                     + (value.isLocked() ? " [LOCKED]" : ""));
         }
     }
+
+    public static void move14AndOlderFilesTo15StructureCatched() {
+        try {
+            move14AndOlderFilesTo15Structure();
+        } catch (Throwable t) {
+            OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Critical error during converting old files to new. Continuing");
+            OutputController.getLogger().log(t);
+        }
+
+    }
+
+    private static void move14AndOlderFilesTo15Structure() {
+        int errors = 0;
+        String PRE_15_DEPLOYMENT_DIR = ".icedtea";
+        String LEGACY_USER_HOME = System.getProperty("user.home") + File.separator + PRE_15_DEPLOYMENT_DIR;
+        File configDir = new File(Defaults.USER_CONFIG_HOME);
+        File cacheDir = new File(Defaults.USER_CACHE_HOME);
+        File legacyUserDir = new File(LEGACY_USER_HOME);
+        if (legacyUserDir.exists()) {
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Legacy configuration and cache found. Those will be now transported to new locations");
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, Defaults.USER_CONFIG_HOME + " and " + Defaults.USER_CACHE_HOME);
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "You should not see this message next time you run icedtea-web!");
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Your custom dirs will not be touched and will work");
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "-----------------------------------------------");
+
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Preparing new directories:");
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, " " + Defaults.USER_CONFIG_HOME);
+            errors += resultToStd(configDir.mkdirs());
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, " " + Defaults.USER_CACHE_HOME);
+            errors += resultToStd(cacheDir.mkdirs());
+
+            String legacySecurity = LEGACY_USER_HOME + File.separator + "security";
+            String currentSecurity = Defaults.USER_SECURITY;
+            errors += moveLegacyToCurrent(legacySecurity, currentSecurity);
+
+            String legacyCache = LEGACY_USER_HOME + File.separator + "cache";
+            String currentCache = Defaults.getDefaults().get(DeploymentConfiguration.KEY_USER_CACHE_DIR).getDefaultValue();
+            errors += moveLegacyToCurrent(legacyCache, currentCache);
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Adapting " + CacheLRUWrapper.CACHE_INDEX_FILE_NAME + " to new destination");
+            //replace all legacyCache by currentCache in new recently_used
+            try {
+                File f = new File(currentCache, CacheLRUWrapper.CACHE_INDEX_FILE_NAME);
+                String s = FileUtils.loadFileAsString(f);
+                s = s.replace(legacyCache, currentCache);
+                FileUtils.saveFile(s, f);
+            } catch (IOException ex) {
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, ex);
+                errors++;
+            }
+
+            String legacyPcahceDir = LEGACY_USER_HOME + File.separator + "pcache";
+            String currentPcacheDir = Defaults.getDefaults().get(DeploymentConfiguration.KEY_USER_PERSISTENCE_CACHE_DIR).getDefaultValue();
+            errors += moveLegacyToCurrent(legacyPcahceDir, currentPcacheDir);
+
+            String legacyLogDir = LEGACY_USER_HOME + File.separator + "log";
+            String currentLogDir = Defaults.getDefaults().get(DeploymentConfiguration.KEY_USER_LOG_DIR).getDefaultValue();
+            errors += moveLegacyToCurrent(legacyLogDir, currentLogDir);
+
+            String legacyProperties = LEGACY_USER_HOME + File.separator + DEPLOYMENT_PROPERTIES;
+            String currentProperties = Defaults.USER_CONFIG_HOME + File.separator + DEPLOYMENT_PROPERTIES;
+            errors += moveLegacyToCurrent(legacyProperties, currentProperties);
+
+            String legacyPropertiesOld = LEGACY_USER_HOME + File.separator + DEPLOYMENT_PROPERTIES + ".old";
+            String currentPropertiesOld = Defaults.USER_CONFIG_HOME + File.separator + DEPLOYMENT_PROPERTIES + ".old";
+            errors += moveLegacyToCurrent(legacyPropertiesOld, currentPropertiesOld);
+
+
+            String legacyAppletTrust = LEGACY_USER_HOME + File.separator + APPLET_TRUST_SETTINGS;
+            String currentAppletTrust = getAppletTrustUserSettingsPath().getAbsolutePath();
+            errors += moveLegacyToCurrent(legacyAppletTrust, currentAppletTrust);
+
+            String legacyTmp = LEGACY_USER_HOME + File.separator + "tmp";
+            String currentTmp = Defaults.getDefaults().get(DeploymentConfiguration.KEY_USER_TMP_DIR).getDefaultValue();
+            errors += moveLegacyToCurrent(legacyTmp, currentTmp);
+
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Removing now empty " + LEGACY_USER_HOME);
+            errors += resultToStd(legacyUserDir.delete());
+
+            if (errors != 0) {
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "There occureed " + errors + " errors");
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Please double check content of old data in " + LEGACY_USER_HOME + " with ");
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "new " + Defaults.USER_CONFIG_HOME + " and " + Defaults.USER_CACHE_HOME);
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "To disable this check again, please remove " + LEGACY_USER_HOME);
+            }
+
+        } else {
+            OutputController.getLogger().log("System is already following XDG .cache and .config specifications");
+            try {
+                OutputController.getLogger().log("config: " + Defaults.USER_CONFIG_HOME + " file exists: " + configDir.exists());
+            } catch (Exception ex) {
+                OutputController.getLogger().log(ex);
+            }
+            try {
+                OutputController.getLogger().log("cache: " + Defaults.USER_CACHE_HOME + " file exists:" + cacheDir.exists());
+            } catch (Exception ex) {
+                OutputController.getLogger().log(ex);
+            }
+        }
+        //this call should endure even if (ever) will migration code be removed
+        DirectoryValidator.DirectoryCheckResults r = new DirectoryValidator().ensureDirs();
+        if (!JNLPRuntime.isHeadless()) {
+            if (r.getFailures() > 0) {
+                JOptionPane.showMessageDialog(null, r.getMessage());
+            }
+        }
+
+    }
+
+    private static int moveLegacyToCurrent(String legacy, String current) {
+        OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Moving " + legacy + " to " + current);
+        File cf = new File(current);
+        File old = new File(legacy);
+        if (cf.exists()) {
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Warning! Destination " + current + " exists!");
+        }
+        if (old.exists()) {
+            boolean moved = old.renameTo(cf);
+            return resultToStd(moved);
+        } else {
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Source " + legacy + " do not exists, nothing to do");
+            return 0;
+        }
+
+    }
+
+    private static int resultToStd(boolean securityMove) {
+        if (securityMove) {
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "OK");
+            return 0;
+        } else {
+            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "ERROR");
+            return 1;
+        }
+    }
 }
