@@ -39,7 +39,6 @@ import java.awt.BorderLayout;
 import java.awt.Component;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.awt.event.KeyAdapter;
 import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.io.BufferedWriter;
@@ -76,11 +75,12 @@ import javax.swing.table.TableRowSorter;
 import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.Translator;
 import net.sourceforge.jnlp.security.appletextendedsecurity.AppletSecurityLevel;
-import net.sourceforge.jnlp.security.appletextendedsecurity.ExecuteUnsignedApplet;
+import net.sourceforge.jnlp.security.appletextendedsecurity.ExecuteAppletAction;
 import net.sourceforge.jnlp.security.appletextendedsecurity.ExtendedAppletSecurityHelp;
 import net.sourceforge.jnlp.security.appletextendedsecurity.UnsignedAppletActionEntry;
 import net.sourceforge.jnlp.security.appletextendedsecurity.UrlRegEx;
 import net.sourceforge.jnlp.security.appletextendedsecurity.impl.UnsignedAppletActionStorageExtendedImpl;
+import net.sourceforge.jnlp.util.logging.OutputController;
 import net.sourceforge.jnlp.util.ScreenFinder;
 
 public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
@@ -116,7 +116,6 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
     private UnsignedAppletActionTableModel currentModel;
     private String lastDoc;
     private String lastCode;
-    private final UnsignedAppletsTrustingListPanel self;
 
 
     /*
@@ -143,14 +142,13 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
                     f.add(new UnsignedAppletsTrustingListPanel(ff2, ff1, cc));
                     f.setVisible(true);
                 } catch (Exception ex) {
-                    ex.printStackTrace();
+                    OutputController.getLogger().log(OutputController.Level.ERROR_ALL, ex);
                 }
             }
         });
     }
 
     public UnsignedAppletsTrustingListPanel(File globalSettings, File customSettings, DeploymentConfiguration conf) {
-        self = this;
         customBackEnd = new UnsignedAppletActionStorageExtendedImpl(customSettings);
         globalBackEnd = new UnsignedAppletActionStorageExtendedImpl(globalSettings);
         customModel = new UnsignedAppletActionTableModel(customBackEnd);
@@ -511,7 +509,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
             conf.setProperty(DeploymentConfiguration.KEY_SECURITY_LEVEL, ((AppletSecurityLevel) mainPolicyComboBox.getSelectedItem()).toChars());
             conf.save();
         } catch (Exception ex) {
-            ex.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, ex);
             JOptionPane.showMessageDialog(this, ex);
         }
     }
@@ -522,16 +520,16 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
             removeSelectedFromTable(currentTable);
         }
         if (deleteTypeComboBox.getSelectedIndex() == 1) {
-            removeByBehaviour(ExecuteUnsignedApplet.ALWAYS);
+            removeByBehaviour(ExecuteAppletAction.ALWAYS);
         }
         if (deleteTypeComboBox.getSelectedIndex() == 2) {
-            removeByBehaviour(ExecuteUnsignedApplet.NEVER);
+            removeByBehaviour(ExecuteAppletAction.NEVER);
         }
         if (deleteTypeComboBox.getSelectedIndex() == 3) {
-            removeByBehaviour(ExecuteUnsignedApplet.YES);
+            removeByBehaviour(ExecuteAppletAction.YES);
         }
         if (deleteTypeComboBox.getSelectedIndex() == 4) {
-            removeByBehaviour(ExecuteUnsignedApplet.NO);
+            removeByBehaviour(ExecuteAppletAction.NO);
         }
         if (deleteTypeComboBox.getSelectedIndex() == 5) {
             removeAllItemsFromTable(currentTable, customModel);
@@ -552,7 +550,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
                 JOptionPane.showMessageDialog(this, Translator.R("APPEXTSECguiPanelMatchingNote") + "\n" + appletItemsToCaption(i, Translator.R("APPEXTSECguiPanelMatched") + ": "));
             }
         } catch (Exception ex) {
-            ex.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, ex);
             JOptionPane.showMessageDialog(this, Translator.R("APPEXTSECguiPanelMatchingError", ex));
         }
 
@@ -569,7 +567,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
         try {
             f = File.createTempFile("appletTable", "validation");
         } catch (Exception ex) {
-            ex.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, ex);
             JOptionPane.showMessageDialog(this, Translator.R("APPEXTSECguiPanelCanNOtValidate", ex.toString()));
             return;
         }
@@ -600,7 +598,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
             }
             JOptionPane.showMessageDialog(this, Translator.R("APPEXTSECguiPanelTableValid"));
         } catch (Exception ex) {
-            ex.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, ex);
             JOptionPane.showMessageDialog(this, Translator.R("APPEXTSECguiPanelTableInvalid ", ex.toString()));
         } finally {
             f.delete();
@@ -645,7 +643,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
                 break;
             }
         }
-        //System.out.println(+orig+" "+i+" "+nwx+" "+nw+" ");
+        //ItwLogger.getLogger().log(OutputController.Level.ERROR_ALL, orig+" "+i+" "+nwx+" "+nw+" ");
         if (nw != orig) {
             if (orig >= 1) {
                 currentTable.getSelectionModel().setSelectionInterval(orig - 1, orig - 1);
@@ -673,7 +671,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
                 break;
             }
         }
-        // System.out.println(+orig+" "+i+" "+nwx+" "+nw+" ");
+        // OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, orig+" "+i+" "+nwx+" "+nw+" ");
         if (nw != orig) {
             if (orig < currentModel.getRowCount()) {
                 currentTable.getSelectionModel().setSelectionInterval(orig + 1, orig + 1);
@@ -703,7 +701,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
             public TableCellEditor getCellEditor(int row, int column) {
                 int columnx = convertColumnIndexToModel(column);
                 if (columnx == 0) {
-                    return new DefaultCellEditor(new JComboBox(new ExecuteUnsignedApplet[]{ExecuteUnsignedApplet.ALWAYS, ExecuteUnsignedApplet.NEVER, ExecuteUnsignedApplet.YES, ExecuteUnsignedApplet.NO}));
+                    return new DefaultCellEditor(new JComboBox(new ExecuteAppletAction[]{ExecuteAppletAction.ALWAYS, ExecuteAppletAction.NEVER, ExecuteAppletAction.YES, ExecuteAppletAction.NO}));
                 }
                 if (columnx == 2) {
                     column = convertColumnIndexToModel(column);
@@ -760,10 +758,10 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
 
     }
 
-    private void removeByBehaviour(ExecuteUnsignedApplet unsignedAppletAction) {
+    private void removeByBehaviour(ExecuteAppletAction unsignedAppletAction) {
         UnsignedAppletActionEntry[] items = currentModel.back.toArray();
         if (askBeforeActionCheckBox.isSelected()) {
-            List<UnsignedAppletActionEntry> toBeDeleted = new ArrayList();
+            List<UnsignedAppletActionEntry> toBeDeleted = new ArrayList<UnsignedAppletActionEntry>();
             for (int i = 0; i < items.length; i++) {
                 UnsignedAppletActionEntry unsignedAppletActionEntry = items[i];
                 if (unsignedAppletActionEntry.getUnsignedAppletAction() == unsignedAppletAction) {
@@ -879,7 +877,7 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
         @Override
         public void keyPressed(KeyEvent e) {
             if (e.getKeyCode() == KeyEvent.VK_DELETE && !currentModel.back.isReadOnly()) {
-                removeSelectedFromTable(table, askBeforeActionCheckBox.isSelected(), (UnsignedAppletActionTableModel) table.getModel(), self);
+                removeSelectedFromTable(table, askBeforeActionCheckBox.isSelected(), (UnsignedAppletActionTableModel) table.getModel(), UnsignedAppletsTrustingListPanel.this);
             }
         }
 
@@ -906,16 +904,16 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
 
             @Override
             public boolean include(Entry<? extends UnsignedAppletActionTableModel, ? extends Integer> entry) {
-                ExecuteUnsignedApplet o = (ExecuteUnsignedApplet) entry.getModel().getValueAt(entry.getIdentifier(), 0);
-                return (o.equals(ExecuteUnsignedApplet.ALWAYS) || o.equals(ExecuteUnsignedApplet.NEVER));
+                ExecuteAppletAction o = (ExecuteAppletAction) entry.getModel().getValueAt(entry.getIdentifier(), 0);
+                return (o.equals(ExecuteAppletAction.ALWAYS) || o.equals(ExecuteAppletAction.NEVER));
             }
         }
 
         private static final class ShowPermanentA extends MyCommonSorter {
             @Override
             public boolean include(Entry<? extends UnsignedAppletActionTableModel, ? extends Integer> entry) {
-                ExecuteUnsignedApplet o = (ExecuteUnsignedApplet) entry.getModel().getValueAt(entry.getIdentifier(), 0);
-                return (o.equals(ExecuteUnsignedApplet.ALWAYS));
+                ExecuteAppletAction o = (ExecuteAppletAction) entry.getModel().getValueAt(entry.getIdentifier(), 0);
+                return (o.equals(ExecuteAppletAction.ALWAYS));
             }
         }
 
@@ -923,8 +921,8 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
 
             @Override
             public boolean include(Entry<? extends UnsignedAppletActionTableModel, ? extends Integer> entry) {
-                ExecuteUnsignedApplet o = (ExecuteUnsignedApplet) entry.getModel().getValueAt(entry.getIdentifier(), 0);
-                return (o.equals(ExecuteUnsignedApplet.NEVER));
+                ExecuteAppletAction o = (ExecuteAppletAction) entry.getModel().getValueAt(entry.getIdentifier(), 0);
+                return (o.equals(ExecuteAppletAction.NEVER));
             }
         }
 
@@ -932,8 +930,8 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
 
             @Override
             public boolean include(Entry<? extends UnsignedAppletActionTableModel, ? extends Integer> entry) {
-                ExecuteUnsignedApplet o = (ExecuteUnsignedApplet) entry.getModel().getValueAt(entry.getIdentifier(), 0);
-                return (o.equals(ExecuteUnsignedApplet.YES) || o.equals(ExecuteUnsignedApplet.NO));
+                ExecuteAppletAction o = (ExecuteAppletAction) entry.getModel().getValueAt(entry.getIdentifier(), 0);
+                return (o.equals(ExecuteAppletAction.YES) || o.equals(ExecuteAppletAction.NO));
             }
         }
 
@@ -941,8 +939,8 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
 
             @Override
             public boolean include(Entry<? extends UnsignedAppletActionTableModel, ? extends Integer> entry) {
-                ExecuteUnsignedApplet o = (ExecuteUnsignedApplet) entry.getModel().getValueAt(entry.getIdentifier(), 0);
-                return (o.equals(ExecuteUnsignedApplet.YES));
+                ExecuteAppletAction o = (ExecuteAppletAction) entry.getModel().getValueAt(entry.getIdentifier(), 0);
+                return (o.equals(ExecuteAppletAction.YES));
             }
            
             
@@ -952,8 +950,8 @@ public class UnsignedAppletsTrustingListPanel extends javax.swing.JPanel {
 
             @Override
             public boolean include(Entry<? extends UnsignedAppletActionTableModel, ? extends Integer> entry) {
-                ExecuteUnsignedApplet o = (ExecuteUnsignedApplet) entry.getModel().getValueAt(entry.getIdentifier(), 0);
-                return (o.equals(ExecuteUnsignedApplet.NO));
+                ExecuteAppletAction o = (ExecuteAppletAction) entry.getModel().getValueAt(entry.getIdentifier(), 0);
+                return (o.equals(ExecuteAppletAction.NO));
             }
             
         }
