@@ -21,9 +21,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.PrintStream;
 import java.lang.reflect.Constructor;
-import java.lang.reflect.Method;
 import java.lang.reflect.InvocationTargetException;
 import java.net.Authenticator;
 import java.net.ProxySelector;
@@ -33,8 +31,6 @@ import java.security.AllPermission;
 import java.security.KeyStore;
 import java.security.Policy;
 import java.security.Security;
-import java.security.cert.CertificateException;
-import java.security.cert.X509Certificate;
 import java.text.MessageFormat;
 import java.util.List;
 import java.util.ResourceBundle;
@@ -44,10 +40,9 @@ import javax.naming.ConfigurationException;
 import javax.net.ssl.HttpsURLConnection;
 import javax.net.ssl.KeyManagerFactory;
 import javax.net.ssl.SSLContext;
-import javax.net.ssl.SSLEngine;
-import javax.net.ssl.SSLSocket;
 import javax.net.ssl.SSLSocketFactory;
 import javax.net.ssl.TrustManager;
+import javax.swing.JOptionPane;
 import javax.swing.UIManager;
 import javax.swing.text.html.parser.ParserDelegator;
 
@@ -64,39 +59,49 @@ import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.security.JNLPAuthenticator;
 import net.sourceforge.jnlp.security.KeyStores;
 import net.sourceforge.jnlp.security.SecurityDialogMessageHandler;
-import net.sourceforge.jnlp.security.VariableX509TrustManager;
 import net.sourceforge.jnlp.services.XServiceManagerStub;
 import net.sourceforge.jnlp.util.FileUtils;
-import net.sourceforge.jnlp.util.TeeOutputStream;
+import net.sourceforge.jnlp.util.logging.JavaConsole;
+import net.sourceforge.jnlp.util.logging.OutputController;
+import net.sourceforge.jnlp.util.logging.LogConfig;
 import sun.net.www.protocol.jar.URLJarFile;
 
 /**
+ * <p>
  * Configure and access the runtime environment.  This class
  * stores global jnlp properties such as default download
  * indicators, the install/base directory, the default resource
  * update policy, etc.  Some settings, such as the base directory,
- * cannot be changed once the runtime has been initialized.<p>
- *
+ * cannot be changed once the runtime has been initialized.
+ * </p>
+ * <p>
  * The JNLP runtime can be locked to prevent further changes to
  * the runtime environment except by a specified class.  If set,
  * only instances of the <i>exit class</i> can exit the JVM or
  * change the JNLP runtime settings once the runtime has been
- * initialized.<p>
+ * initialized.
+ * </p>
  *
  * @author <a href="mailto:jmaxwell@users.sourceforge.net">Jon A. Maxwell (JAM)</a> - initial author
  * @version $Revision: 1.19 $
  */
 public class JNLPRuntime {
-    
+
     static {
         loadResources();
     }
 
+    /**
+     * java-abrt-connector can print out specific application String method, it is good to save visited urls for reproduce purposes.
+     * For javaws we can read the destination jnlp from commandline
+     * However for plugin (url arrive via pipes). Also for plugin we can not be sure which opened tab/window
+     * have caused the crash. Thats why the individual urls are added, not replaced.
+     */
+    private static String history = "";
+
     /** the localized resource strings */
     private static ResourceBundle resources;
 
-    private static final DeploymentConfiguration config = new DeploymentConfiguration();
-
     /** the security manager */
     private static JNLPSecurityManager security;
 
@@ -130,8 +135,10 @@ public class JNLPRuntime {
     /** whether debug mode is on */
     private static boolean debug = false;
 
-    /** whether streams should be redirected */
-    private static boolean redirectStreams = false;
+    /**
+     * whether plugin debug mode is on
+     */
+    private static Boolean pluginDebug = null;
 
     /** mutex to wait on, for initialization */
     public static Object initMutex = new Object();
@@ -144,12 +151,20 @@ public class JNLPRuntime {
 
     /** all security dialogs will be consumed and pretented as being verified by user and allowed.*/
     private static boolean trustAll=false;
+
+    /** all security dialogs will be consumed and we will pretend the Sandbox option was chosen */
+    private static boolean trustNone = false;
+    
+    /** allows 301.302.303.307.308 redirects to be followed when downloading resources*/
+    private static boolean allowRedirect = false;;
+
     /** 
-     * Header is not checked and so eg. gifar exploit is possible
-     * @see http://en.wikipedia.org/wiki/Gifar for this kind of attack.
+     * Header is not checked and so eg
+     * <a href="https://en.wikipedia.org/wiki/Gifar">gifar</a> exploit is
+     * possible.<br/>
      * However if jar file is a bit corrupted, then it sometimes can work so 
      * this switch can disable the header check.
-     * 
+     * @see <a href="https://en.wikipedia.org/wiki/Gifar">Gifar attack</a>
      */
     private static boolean ignoreHeaders=false;
 
@@ -159,14 +174,10 @@ public class JNLPRuntime {
     /** a lock which is held to indicate that an instance of netx is running */
     private static FileLock fileLock;
 
-    public static final String STDERR_FILE = "java.stderr";
-    public static final String STDOUT_FILE = "java.stdout";
-
-
     /**
      * Returns whether the JNLP runtime environment has been
-     * initialized.  Once initialized, some properties such as the
-     * base directory cannot be changed.  Before
+     * initialized. Once initialized, some properties such as the
+     * base directory cannot be changed. Before
      */
     public static boolean isInitialized() {
         return initialized;
@@ -175,35 +186,43 @@ public class JNLPRuntime {
     /**
      * Initialize the JNLP runtime environment by installing the
      * security manager and security policy, initializing the JNLP
-     * standard services, etc.<p>
-     *
-     * This method should be called from the main AppContext/Thread. <p>
-     *
-     * This method cannot be called more than once.  Once
+     * standard services, etc.
+     * <p>
+     * This method should be called from the main AppContext/Thread.
+     * </p>
+     * <p>
+     * This method cannot be called more than once. Once
      * initialized, methods that alter the runtime can only be
-     * called by the exit class.<p>
-     *
-     * @param isApplication is true if a webstart application is being initialized
+     * called by the exit class.
+     * </p>
      *
+     * @param isApplication is {@code true} if a webstart application is being
+     * initialized
      * @throws IllegalStateException if the runtime was previously initialized
      */
     public static void initialize(boolean isApplication) throws IllegalStateException {
         checkInitialized();
 
         try {
-            config.load();
-            config.copyTo(System.getProperties());
-        } catch (ConfigurationException e) {
-            /* exit if there is a fatal exception loading the configuration */
-            if (isApplication) {
-                System.out.println(getMessage("RConfigurationError"));
-                System.exit(1);
-            }
+            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
+        } catch (Exception e) {
+            OutputController.getLogger().log("Unable to set system look and feel");
         }
 
-        KeyStores.setConfiguration(config);
-
-        initializeStreams();
+        if (JavaConsole.canShowOnStartup(isApplication)) {
+            JavaConsole.getConsole().showConsoleLater();
+        }
+        /* exit if there is a fatal exception loading the configuration */
+        if (getConfiguration().getLoadingException() != null) {
+            if (getConfiguration().getLoadingException() instanceof ConfigurationException){
+                // ConfigurationException is thrown only if deployment.config's field
+                // deployment.system.config.mandatory is true, and the destination
+                //where deployment.system.config points is not readable
+                throw new RuntimeException(getConfiguration().getLoadingException());
+            }
+            OutputController.getLogger().log(OutputController.Level.WARNING_ALL, getMessage("RConfigurationError")+": "+getConfiguration().getLoadingException().getMessage());
+        }
+        KeyStores.setConfiguration(getConfiguration());
 
         isWebstartApplication = isApplication;
 
@@ -220,9 +239,9 @@ public class JNLPRuntime {
 
         if (handler == null) {
             if (headless) {
-                handler = new DefaultLaunchHandler(System.err);
+                handler = new DefaultLaunchHandler(OutputController.getLogger());
             } else {
-                handler = new GuiLaunchHandler(System.err);
+                handler = new GuiLaunchHandler(OutputController.getLogger());
             }
         }
 
@@ -231,12 +250,6 @@ public class JNLPRuntime {
         policy = new JNLPPolicy();
         security = new JNLPSecurityManager(); // side effect: create JWindow
 
-        try {
-            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
-        } catch (Exception e) {
-            // ignore it
-        }
-
         doMainAppContextHacks();
 
         if (securityEnabled) {
@@ -259,13 +272,15 @@ public class JNLPRuntime {
 
             HttpsURLConnection.setDefaultSSLSocketFactory(sslSocketFactory);
         } catch (Exception e) {
-            System.err.println("Unable to set SSLSocketfactory (may _prevent_ access to sites that should be trusted)! Continuing anyway...");
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Unable to set SSLSocketfactory (may _prevent_ access to sites that should be trusted)! Continuing anyway...");
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, e);
         }
 
         // plug in a custom authenticator and proxy selector
         Authenticator.setDefault(new JNLPAuthenticator());
-        ProxySelector.setDefault(new BrowserAwareProxySelector());
+        BrowserAwareProxySelector proxySelector = new BrowserAwareProxySelector(getConfiguration());
+        proxySelector.initialize();
+        ProxySelector.setDefault(proxySelector);
 
         // Restrict access to netx classes
         Security.setProperty("package.access", 
@@ -277,6 +292,10 @@ public class JNLPRuntime {
 
     }
 
+    public static void reloadPolicy() {
+        policy.refresh();
+    }
+
     /**
      * Returns a TrustManager ideal for the running VM.
      *
@@ -294,14 +313,14 @@ public class JNLPRuntime {
                 try {
                     trustManagerClass = Class.forName("net.sourceforge.jnlp.security.VariableX509TrustManagerJDK6");
                  } catch (ClassNotFoundException cnfe) {
-                     System.err.println("Unable to find class net.sourceforge.jnlp.security.VariableX509TrustManagerJDK6");
+                     OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Unable to find class net.sourceforge.jnlp.security.VariableX509TrustManagerJDK6");
                      return null;
                  }
             } else { // Java 7 or more (technically could be <= 1.5 but <= 1.5 is unsupported)
                 try {
                     trustManagerClass = Class.forName("net.sourceforge.jnlp.security.VariableX509TrustManagerJDK7");
                  } catch (ClassNotFoundException cnfe) {
-                     System.err.println("Unable to find class net.sourceforge.jnlp.security.VariableX509TrustManagerJDK7");
+                     OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Unable to find class net.sourceforge.jnlp.security.VariableX509TrustManagerJDK7");
                      return null;
                  }
             }
@@ -318,7 +337,8 @@ public class JNLPRuntime {
 
             return (TrustManager) tmCtor.newInstance();
         } catch (RuntimeException e) {
-            System.err.println("Unable to load JDK-specific TrustManager. Was this version of IcedTea-Web compiled with JDK6?");
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Unable to load JDK-specific TrustManager. Was this version of IcedTea-Web compiled with JDK 6 or 7?");
+            OutputController.getLogger().log(e);
             throw e;
         }
     }
@@ -357,44 +377,51 @@ public class JNLPRuntime {
         new ParserDelegator();
     }
 
+   
     /**
-     * Initializes the standard output and error streams, redirecting them or
-     * duplicating them as required.
+     * see <a href="https://en.wikipedia.org/wiki/Double-checked_locking#Usage_in_Java">Double-checked locking in Java</a>
+     * for cases how not to do lazy initialization
+     * and <a href="https://en.wikipedia.org/wiki/Initialization_on_demand_holder_idiom">Initialization on demand holder idiom</a>
+     * for ITW approach
      */
-    private static void initializeStreams() {
-        Boolean enableLogging = Boolean.valueOf(config
-                .getProperty(DeploymentConfiguration.KEY_ENABLE_LOGGING));
-        if (redirectStreams || enableLogging) {
-            String logDir = config.getProperty(DeploymentConfiguration.KEY_USER_LOG_DIR);
+    private static class DeploymentConfigurationHolder {
 
+        private static final DeploymentConfiguration INSTANCE = initConfiguration();
+
+        private static DeploymentConfiguration initConfiguration() {
+            DeploymentConfiguration config = new DeploymentConfiguration();
             try {
-                File errFile = new File(logDir, JNLPRuntime.STDERR_FILE);
-                FileUtils.createParentDir(errFile);
-                FileUtils.createRestrictedFile(errFile, true);
-                File outFile = new File(logDir, JNLPRuntime.STDOUT_FILE);
-                FileUtils.createParentDir(outFile);
-                FileUtils.createRestrictedFile(outFile, true);
-
-                if (redirectStreams) {
-                    System.setErr(new PrintStream(new FileOutputStream(errFile)));
-                    System.setOut(new PrintStream(new FileOutputStream(outFile)));
-                } else {
-                    System.setErr(new TeeOutputStream(new FileOutputStream(errFile), System.err));
-                    System.setOut(new TeeOutputStream(new FileOutputStream(outFile), System.out));
+                config.load();
+                config.copyTo(System.getProperties());
+            } catch (ConfigurationException ex) {
+                OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, Translator.R("RConfigurationError"));
+                //mark this exceptionas we can die on it later
+                config.setLoadingException(ex);
+                //to be sure - we MUST die - http://docs.oracle.com/javase/6/docs/technotes/guides/deployment/deployment-guide/properties.html
+            }catch(Exception t){
+                //all exceptions are causing InstantiatizationError so this do it much more readble
+                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, t);
+                OutputController.getLogger().log(OutputController.Level.WARNING_ALL, Translator.R("RFailingToDefault"));
+                if (!JNLPRuntime.isHeadless()){
+                    JOptionPane.showMessageDialog(null, getMessage("RFailingToDefault")+"\n"+t.toString());
                 }
-            } catch (Exception e) {
-                e.printStackTrace();
+                //try to survive this unlikely exception
+                config.resetToDefaults();
+            } finally {
+                OutputController.getLogger().startConsumer();
             }
+            return config;
         }
     }
 
     /**
      * Gets the Configuration associated with this runtime
+     *
      * @return a {@link DeploymentConfiguration} object that can be queried to
      * find relevant configuration settings
      */
     public static DeploymentConfiguration getConfiguration() {
-        return config;
+        return DeploymentConfigurationHolder.INSTANCE;
     }
 
     /**
@@ -424,7 +451,7 @@ public class JNLPRuntime {
      * Sets whether the JNLP client will use any AWT/Swing
      * components.  In headless mode, client features that use the
      * AWT are disabled such that the client can be used in
-     * headless mode (<code>java.awt.headless=true</code>).
+     * headless mode ({@code java.awt.headless=true}).
      *
      * @throws IllegalStateException if the runtime was previously initialized
      */
@@ -432,6 +459,16 @@ public class JNLPRuntime {
         checkInitialized();
         headless = enabled;
     }
+    
+    public static void setAllowRedirect(boolean enabled) {
+        checkInitialized();
+        allowRedirect = enabled;
+    }
+
+    public static boolean isAllowRedirect() {
+        return allowRedirect;
+    }
+    
 
     /**
          * Sets whether we will verify code signing.
@@ -454,10 +491,11 @@ public class JNLPRuntime {
      * Disabling security can increase performance for some
      * applications, and can be used to use netx with other code
      * that uses its own security manager or policy.
-     *
+     * <p>
      * Disabling security is not recommended and should only be
-     * used if the JNLP files opened are trusted.  This method can
-     * only be called before initalizing the runtime.<p>
+     * used if the JNLP files opened are trusted. This method can
+     * only be called before initalizing the runtime.
+     * </p>
      *
      * @param enabled whether security should be enabled
      * @throws IllegalStateException if the runtime is already initialized
@@ -486,7 +524,7 @@ public class JNLPRuntime {
      *
      * @throws IllegalStateException if caller is not the exit class
      */
-    public static void setExitClass(Class exitClass) {
+    public static void setExitClass(Class<?> exitClass) {
         checkExitClass();
         security.setExitClass(exitClass);
     }
@@ -513,6 +551,10 @@ public class JNLPRuntime {
      * should be printed.
      */
     public static boolean isDebug() {
+        return isSetDebug() ||  isPluginDebug() || LogConfig.getLogConfig().isEnableLogging();
+    }
+
+     public static boolean isSetDebug() {
         return debug;
     }
 
@@ -527,17 +569,7 @@ public class JNLPRuntime {
         debug = enabled;
     }
 
-    /**
-     * Sets whether the standard output/error streams should be redirected to
-     * the loggging files.
-     *
-     * @throws IllegalStateException if the runtime has already been initialized
-     */
-    public static void setRedirectStreams(boolean redirect) {
-        checkInitialized();
-        redirectStreams = redirect;
-    }
-
+  
     /**
      * Sets the default update policy.
      *
@@ -589,7 +621,7 @@ public class JNLPRuntime {
 
     /**
      * Returns the localized resource string identified by the
-     * specified key.  If the message is empty, a null is
+     * specified key. If the message is empty, a null is
      * returned.
      */
     public static String getMessage(String key) {
@@ -608,8 +640,7 @@ public class JNLPRuntime {
     }
 
     /**
-     * Returns the localized resource string using the specified
-     * arguments.
+     * Returns the localized resource string using the specified arguments.
      *
      * @param args the formatting arguments to the resource string
      */
@@ -618,7 +649,7 @@ public class JNLPRuntime {
     }
 
     /**
-     * Returns true if the current runtime will fork
+     * Returns {@code true} if the current runtime will fork
      */
     public static boolean getForksAllowed() {
         return forksAllowed;
@@ -630,8 +661,7 @@ public class JNLPRuntime {
     }
 
     /**
-     * Throws an exception if called when the runtime is
-     * already initialized.
+     * Throws an exception if called when the runtime is already initialized.
      */
     private static void checkInitialized() {
         if (initialized)
@@ -639,9 +669,8 @@ public class JNLPRuntime {
     }
 
     /**
-     * Throws an exception if called with security enabled but
-     * a caller is not the exit class and the runtime has been
-     * initialized.
+     * Throws an exception if called with security enabled but a caller is not
+     * the exit class and the runtime has been initialized.
      */
     private static void checkExitClass() {
         if (securityEnabled && initialized)
@@ -674,7 +703,7 @@ public class JNLPRuntime {
     }
 
     /**
-     * @return true if running on Windows
+     * @return {@code true} if running on Windows
      */
     public static boolean isWindows() {
         String os = System.getProperty("os.name");
@@ -682,8 +711,8 @@ public class JNLPRuntime {
     }
 
     /**
-     * @return true if running on a Unix or Unix-like system (including Linux
-     * and *BSD)
+     * @return {@code true} if running on a Unix or Unix-like system (including
+     * Linux and *BSD)
      */
     public static boolean isUnix() {
         String sep = System.getProperty("file.separator");
@@ -703,7 +732,8 @@ public class JNLPRuntime {
     }
 
     /**
-     * Indicate that netx is running by creating the {@link JNLPRuntime#INSTANCE_FILE} and
+     * Indicate that netx is running by creating the
+     * {@link DeploymentConfiguration#KEY_USER_NETX_RUNNING_FILE} and
      * acquiring a shared lock on it
      */
     public synchronized static void markNetxRunning() {
@@ -737,13 +767,11 @@ public class JNLPRuntime {
             }
             
             if (fileLock != null && fileLock.isShared()) {
-                if (JNLPRuntime.isDebug()) {
-                    System.out.println("Acquired shared lock on " +
+                OutputController.getLogger().log("Acquired shared lock on " +
                             netxRunningFile.toString() + " to indicate javaws is running");
-                }
             }
         } catch (IOException e) {
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, e);
         }
 
         Runtime.getRuntime().addShutdownHook(new Thread("JNLPRuntimeShutdownHookThread") {
@@ -756,7 +784,7 @@ public class JNLPRuntime {
 
     /**
      * Indicate that netx is stopped by releasing the shared lock on
-     * {@link JNLPRuntime#INSTANCE_FILE}.
+     * {@link DeploymentConfiguration#KEY_USER_NETX_RUNNING_FILE}.
      */
     private static void markNetxStopped() {
         if (fileLock == null) {
@@ -766,13 +794,10 @@ public class JNLPRuntime {
             fileLock.release();
             fileLock.channel().close();
             fileLock = null;
-            if (JNLPRuntime.isDebug()) {
-                String file = JNLPRuntime.getConfiguration()
-                        .getProperty(DeploymentConfiguration.KEY_USER_NETX_RUNNING_FILE);
-                System.out.println("Release shared lock on " + file);
-            }
+            OutputController.getLogger().log("Release shared lock on " + JNLPRuntime.getConfiguration()
+                        .getProperty(DeploymentConfiguration.KEY_USER_NETX_RUNNING_FILE));
         } catch (IOException e) {
-            e.printStackTrace();
+            OutputController.getLogger().log(e);
         }
     }
 
@@ -784,6 +809,14 @@ public class JNLPRuntime {
         return trustAll;
     }
 
+    static void setTrustNone(final boolean b) {
+        trustNone = b;
+    }
+
+    public static boolean isTrustNone() {
+        return trustNone;
+    }
+
     public static boolean isIgnoreHeaders() {
         return ignoreHeaders;
     }
@@ -791,6 +824,38 @@ public class JNLPRuntime {
     public static void setIgnoreHeaders(boolean ignoreHeaders) {
         JNLPRuntime.ignoreHeaders = ignoreHeaders;
     }
+
+    private static boolean isPluginDebug() {
+        if (pluginDebug == null) {
+            try {
+                //there are cases when this itself is not allowed by security manager, and so
+                //throws exception. Under some conditions it can couse deadlock
+                pluginDebug = System.getenv().containsKey("ICEDTEAPLUGIN_DEBUG");
+            } catch (Exception ex) {
+                pluginDebug = false;
+                OutputController.getLogger().log(ex);
+            }
+        }
+        return pluginDebug;
+    }
+
+    public static void exit(int i) {
+        OutputController.getLogger().close();
+        System.exit(i);
+    }
+
+
+    public static void saveHistory(String documentBase) {
+        JNLPRuntime.history += " " + documentBase + " ";
+    }
+
+    /**
+     * Used by java-abrt-connector via reflection
+     * @return history
+     */
+    private static String getHistory() {
+        return history;
+    }
     
     
 
