@@ -42,10 +42,11 @@ import net.sourceforge.jnlp.runtime.ApplicationInstance;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.security.SecurityDialogs;
 import net.sourceforge.jnlp.security.SecurityDialogs.AccessType;
+import net.sourceforge.jnlp.util.logging.OutputController;
 
 /**
  * Provides static methods to interact useful for using the JNLP
- * services.<p>
+ * services.
  *
  * @author <a href="mailto:jmaxwell@users.sourceforge.net">Jon A. Maxwell (JAM)</a> - initial author
  * @author <a href="mailto:jsumali@redhat.com">Joshua Sumali</a>
@@ -154,9 +155,9 @@ public class ServiceUtil {
      * must be more than extremely careful in the operations they
      * perform.
      */
-    static Object createPrivilegedProxy(Class iface, final Object receiver) {
+    static Object createPrivilegedProxy(Class<?> iface, final Object receiver) {
         return java.lang.reflect.Proxy.newProxyInstance(XServiceManagerStub.class.getClassLoader(),
-                new Class[] { iface },
+                new Class<?>[] { iface },
                 new PrivilegedHandler(receiver));
     }
 
@@ -170,15 +171,19 @@ public class ServiceUtil {
             this.receiver = receiver;
         }
 
+        @Override
         public Object invoke(Object proxy, final Method method, final Object[] args) throws Throwable {
             if (JNLPRuntime.isDebug()) {
-                System.err.println("call privileged method: " + method.getName());
-                if (args != null)
-                    for (int i = 0; i < args.length; i++)
-                        System.err.println("           arg: " + args[i]);
+                OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "call privileged method: " + method.getName());
+                if (args != null) {
+                    for (int i = 0; i < args.length; i++) {
+                        OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "           arg: " + args[i]);
+                    }
+                }
             }
 
             PrivilegedExceptionAction<Object> invoker = new PrivilegedExceptionAction<Object>() {
+                @Override
                 public Object run() throws Exception {
                     return method.invoke(receiver, args);
                 }
@@ -187,8 +192,7 @@ public class ServiceUtil {
             try {
                 Object result = AccessController.doPrivileged(invoker);
 
-                if (JNLPRuntime.isDebug())
-                    System.err.println("        result: " + result);
+                OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "        result: " + result);
 
                 return result;
             } catch (PrivilegedActionException e) {
@@ -235,41 +239,16 @@ public class ServiceUtil {
     public static boolean checkAccess(ApplicationInstance app, AccessType type,
                 Object... extras) {
 
-        if (app == null)
-            app = JNLPRuntime.getApplication();
+        boolean trusted = isSigned(app);
 
-        boolean codeTrusted = true;
-
-        StackTraceElement[] stack = Thread.currentThread().getStackTrace();
-
-        for (int i = 0; i < stack.length; i++) {
-
-            Class c = null;
-
-            try {
-                c = Class.forName(stack[i].getClassName());
-            } catch (Exception e1) {
-                try {
-                    c = Class.forName(stack[i].getClassName(), false, app.getClassLoader());
-                } catch (Exception e2) {
-                    System.err.println(e2.getMessage());
-                }
-            }
-
-            // Everything up to the desired class/method must be trusted
-            if (c == null || // class not found
-                    (c.getProtectionDomain().getCodeSource() != null && // class is not in bootclasspath
-                    c.getProtectionDomain().getCodeSource().getCodeSigners() == null) // class is trusted
-            ) {
-                codeTrusted = false;
-            }
-        }
-
-        if (!codeTrusted) {
+        if (!trusted) {
 
             if (!shouldPromptUser()) {
                 return false;
             }
+            if (app == null) {
+                app = JNLPRuntime.getApplication();
+            }
 
             final AccessType tmpType = type;
             final Object[] tmpExtras = extras;
@@ -279,6 +258,7 @@ public class ServiceUtil {
             //applets, otherwise permissions won't be granted to load icons
             //from resources.jar.
             Boolean b = AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
+                @Override
                 public Boolean run() {
                     boolean b = SecurityDialogs.showAccessWarningDialog(tmpType,
                                 tmpApp.getJNLPFile(), tmpExtras);
@@ -307,5 +287,50 @@ public class ServiceUtil {
             }
         });
     }
+    
+    /**
+     * Returns whether the app requesting a JNLP service is a trusted
+     * application
+     * 
+     * @param app
+     *            the application which is requesting the check. If null, the
+     *            current application is used.
+     * @return true, if the app is a trusted application; false otherwise
+     */
+
+    public static boolean isSigned(ApplicationInstance app) {
+
+        if (app == null) {
+            app = JNLPRuntime.getApplication();
+        }
+
+        StackTraceElement[] stack = Thread.currentThread().getStackTrace();
+
+        for (int i = 0; i < stack.length; i++) {
+
+            Class<?> c = null;
+
+            try {
+                c = Class.forName(stack[i].getClassName());
+            } catch (Exception e1) {
+                OutputController.getLogger().log(e1);
+                try {
+                    c = Class.forName(stack[i].getClassName(), false,
+                            app.getClassLoader());
+                } catch (Exception e2) {
+                    OutputController.getLogger().log(e2);
+                }
+            }
+
+            // Everything up to the desired class/method must be trusted
+            if (c == null || // class not found
+                    (c.getProtectionDomain().getCodeSource() != null && // class is not in bootclasspath
+                    c.getProtectionDomain().getCodeSource().getCodeSigners() == null) // class is trusted
+            ) {
+                return false;
+            }
+        }
+        return true;
+    }
 
 }
