@@ -25,6 +25,7 @@ import net.sourceforge.jnlp.cache.*;
 import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.*;
 import net.sourceforge.jnlp.util.FileUtils;
+import net.sourceforge.jnlp.util.logging.OutputController;
 
 /**
  * The BasicService JNLP service.
@@ -67,10 +68,8 @@ class XPersistenceService implements PersistenceService {
         else
             requestPath = "";
 
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("codebase path: " + source.getFile());
-            System.out.println("request path: " + requestPath);
-        }
+          OutputController.getLogger().log("codebase path: " + source.getFile());
+          OutputController.getLogger().log("request path: " + requestPath);
 
         if (!source.getFile().startsWith(requestPath) 
                 && !ServiceUtil.isSigned(app)) // Allow trusted application to have access to data below source URL path
