@@ -41,23 +41,23 @@ import java.util.List;
 import java.util.Map;
 import java.util.Vector;
 import java.util.jar.JarEntry;
-import net.sourceforge.jnlp.util.JarFile;
 
 import net.sourceforge.jnlp.JARDesc;
 import net.sourceforge.jnlp.JNLPFile;
 import net.sourceforge.jnlp.LaunchException;
 import net.sourceforge.jnlp.cache.ResourceTracker;
-import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import net.sourceforge.jnlp.runtime.JNLPClassLoader.SecurityDelegate;
 import net.sourceforge.jnlp.security.AppVerifier;
 import net.sourceforge.jnlp.security.CertVerifier;
 import net.sourceforge.jnlp.security.CertificateUtils;
 import net.sourceforge.jnlp.security.KeyStores;
+import net.sourceforge.jnlp.util.JarFile;
+import net.sourceforge.jnlp.util.logging.OutputController;
 import sun.security.util.DerInputStream;
 import sun.security.util.DerValue;
 import sun.security.x509.NetscapeCertTypeExtension;
 
 /**
- * <p>
  * The jar certificate verifier utility.
  * 
  * @author Roland Schemers
@@ -117,19 +117,15 @@ public class JarCertVerifier implements CertVerifier {
     public boolean getAlreadyTrustPublisher() {
         boolean allPublishersTrusted = appVerifier.hasAlreadyTrustedPublisher(
                 certs, jarSignableEntries);
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("App already has trusted publisher: "
+        OutputController.getLogger().log("App already has trusted publisher: "
                     + allPublishersTrusted);
-        }
         return allPublishersTrusted;
     }
 
     public boolean getRootInCacerts() {
         boolean allRootCAsTrusted = appVerifier.hasRootInCacerts(certs,
                 jarSignableEntries);
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("App has trusted root CA: " + allRootCAsTrusted);
-        }
+        OutputController.getLogger().log("App has trusted root CA: " + allRootCAsTrusted);
         return allRootCAsTrusted;
     }
 
@@ -181,13 +177,19 @@ public class JarCertVerifier implements CertVerifier {
             return true;
         boolean fullySigned = appVerifier.isFullySigned(certs,
                 jarSignableEntries);
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("App already has trusted publisher: "
+        OutputController.getLogger().log("App already has trusted publisher: "
                     + fullySigned);
-        }
         return fullySigned;
     }
 
+    public static boolean isJarSigned(JARDesc jar, AppVerifier verifier, ResourceTracker tracker) throws Exception {
+        JarCertVerifier certVerifier = new JarCertVerifier(verifier);
+        List<JARDesc> singleJarList = new ArrayList<JARDesc>();
+        singleJarList.add(jar);
+        certVerifier.add(singleJarList, tracker);
+        return certVerifier.allJarsSigned();
+    }
+
     /**
      * Update the verifier to consider new jars when verifying.
      * 
@@ -292,7 +294,7 @@ public class JarCertVerifier implements CertVerifier {
                     entriesVec);
 
         } catch (Exception e) {
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, e);
             throw e;
         } finally { // close the resource
             if (jarFile != null) {
@@ -418,10 +420,8 @@ public class JarCertVerifier implements CertVerifier {
             result = VerifyResult.UNSIGNED;
         }
 
-        if (JNLPRuntime.isDebug()) {
-            System.out.println("Jar found at " + jarName
+        OutputController.getLogger().log("Jar found at " + jarName
                     + "has been verified as " + result);
-        }
         return result;
     }
 
@@ -452,9 +452,7 @@ public class JarCertVerifier implements CertVerifier {
             // TODO: Warn user about not being able to
             // look through their cacerts/trusted.certs
             // file depending on exception.
-            if (JNLPRuntime.isDebug()) {
-                System.out.println("WARNING: Unable to read through cert store files.");
-            }
+            OutputController.getLogger().log("WARNING: Unable to read through cert store files.");
             throw e;
         }
 
@@ -600,8 +598,8 @@ public class JarCertVerifier implements CertVerifier {
         return this.unverifiedJars.size() == 0;
     }
 
-    public void checkTrustWithUser(JNLPFile file) throws LaunchException {
-        appVerifier.checkTrustWithUser(this, file);
+    public void checkTrustWithUser(SecurityDelegate securityDelegate, JNLPFile file) throws LaunchException {
+        appVerifier.checkTrustWithUser(securityDelegate, this, file);
     }
 
     public Map<String, Integer> getJarSignableEntries() {
