@@ -413,7 +413,7 @@ PluginRequestProcessor::setMember(std::vector<std::string*>* message_parts)
     member = (NPVariant*) (IcedTeaPluginUtilities::stringToJSID(*(message_parts->at(5))));
     propertyNameID = *(message_parts->at(6));
 
-    if (*(message_parts->at(7)) == "literalreturn")
+    if (*(message_parts->at(7)) == "literalreturn" || *(message_parts->at(7)) == "jsobject" )
     {
         value.append(*(message_parts->at(7)));
         value.append(" ");
@@ -440,7 +440,7 @@ PluginRequestProcessor::setMember(std::vector<std::string*>* message_parts)
         // the result we want is in result_string (assuming there was no error)
         if (java_result->error_occurred)
         {
-	    printf("Unable to get member name for setMember. Error occurred: %s\n", java_result->error_msg->c_str());
+	    PLUGIN_ERROR("Unable to get member name for setMember. Error occurred: %s\n", java_result->error_msg->c_str());
             //goto cleanup;
         }
 
@@ -471,7 +471,7 @@ PluginRequestProcessor::setMember(std::vector<std::string*>* message_parts)
  *
  * This is a static function, called in another thread. Since certain data
  * can only be requested from the main thread in Mozilla, this function
- * does whatever it can seperately, and then makes an internal request that
+ * does whatever it can separately, and then makes an internal request that
  * causes _getMember to do the rest of the work.
  *
  * @param message_parts The request message
@@ -521,7 +521,7 @@ PluginRequestProcessor::sendMember(std::vector<std::string*>* message_parts)
         // the result we want is in result_string (assuming there was no error)
         if (java_result->error_occurred)
         {
-	    printf("Unable to process getMember request. Error occurred: %s\n", java_result->error_msg->c_str());
+	    PLUGIN_ERROR("Unable to process getMember request. Error occurred: %s\n", java_result->error_msg->c_str());
             //goto cleanup;
         }
 
@@ -763,7 +763,7 @@ _setMember(void* data)
     else
     	property_identifier = browser_functions.getstringidentifier(property_id->c_str());
 
-    PLUGIN_DEBUG("Setting %s on instance %p, object %p to value %s\n", browser_functions.utf8fromidentifier(property_identifier), instance, member, value->c_str());
+    PLUGIN_DEBUG("Setting %s on instance %p, object %p to value %s\n", IcedTeaPluginUtilities::NPIdentifierAsString(property_identifier).c_str(), instance, member, value->c_str());
 
     IcedTeaPluginUtilities::javaResultToNPVariant(instance, value, &value_variant);
 
@@ -783,6 +783,7 @@ _getMember(void* data)
     std::vector<void*> parameters = ((AsyncCallThreadData*) data)->parameters;
 
     instance = (NPP) parameters.at(0);
+
     parent_ptr = (NPObject*) parameters.at(1);
     std::string*  member_id = (std::string*) parameters.at(2);
     NPIdentifier member_identifier;
@@ -795,11 +796,11 @@ _getMember(void* data)
     	member_identifier = browser_functions.getstringidentifier(member_id->c_str());
 
     // Get the NPVariant corresponding to this member
-    PLUGIN_DEBUG("Looking for %p %p %p (%s)\n", instance, parent_ptr, member_identifier, browser_functions.utf8fromidentifier(member_identifier));
+    PLUGIN_DEBUG("Looking for %p %p %p (%s)\n", instance, parent_ptr, member_identifier, IcedTeaPluginUtilities::NPIdentifierAsString(member_identifier).c_str());
 
     if (!browser_functions.hasproperty(instance, parent_ptr, member_identifier))
     {
-        printf("%s not found!\n", browser_functions.utf8fromidentifier(member_identifier));
+        PLUGIN_ERROR("%s not found!\n", IcedTeaPluginUtilities::NPIdentifierAsString(member_identifier).c_str());
     }
     ((AsyncCallThreadData*) data)->call_successful = browser_functions.getproperty(instance, parent_ptr, member_identifier, member_ptr);
 
@@ -809,7 +810,9 @@ _getMember(void* data)
     {
         createJavaObjectFromVariant(instance, *member_ptr, &member_ptr_str);
         ((AsyncCallThreadData*) data)->result.append(member_ptr_str);
-
+    } else
+    {
+        ((AsyncCallThreadData*) data)->result.append("null");
     }
     ((AsyncCallThreadData*) data)->result_ready = true;
 
@@ -838,17 +841,10 @@ _eval(void* data)
     window_ptr = (NPObject*) call_data->at(1);
     script_str = (std::string*) call_data->at(2);
 
-#if MOZILLA_VERSION_COLLAPSED < 1090200
-    script.utf8characters = script_str->c_str();
-    script.utf8length = script_str->size();
-
-    PLUGIN_DEBUG("Evaluating: %s\n", script_str->c_str());
-#else
     script.UTF8Characters = script_str->c_str();
     script.UTF8Length = script_str->size();
 
     PLUGIN_DEBUG("Evaluating: %s\n", script_str->c_str());
-#endif
 
     ((AsyncCallThreadData*) data)->call_successful = browser_functions.evaluate(instance, window_ptr, &script, eval_variant);
     IcedTeaPluginUtilities::printNPVariant(*eval_variant);
@@ -962,6 +958,9 @@ _getString(void* data)
     if (((AsyncCallThreadData*) data)->call_successful)
     {
         createJavaObjectFromVariant(instance, tostring_result, &(((AsyncCallThreadData*) data)->result));
+    } else
+    {
+        ((AsyncCallThreadData*) data)->result.append("null");
     }
     ((AsyncCallThreadData*) data)->result_ready = true;
 
