@@ -82,8 +82,6 @@ import java.awt.print.Printable;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
-import java.lang.reflect.InvocationTargetException;
-import java.net.MalformedURLException;
 import java.net.SocketPermission;
 import java.net.URI;
 import java.net.URL;
@@ -111,7 +109,6 @@ import net.sourceforge.jnlp.PluginParameters;
 import net.sourceforge.jnlp.runtime.JNLPClassLoader;
 import net.sourceforge.jnlp.security.appletextendedsecurity.AppletSecurityLevel;
 import net.sourceforge.jnlp.security.appletextendedsecurity.AppletStartupSecuritySettings;
-import net.sourceforge.jnlp.security.appletextendedsecurity.ExecuteUnsignedApplet;
 import net.sourceforge.jnlp.splashscreen.SplashController;
 import net.sourceforge.jnlp.splashscreen.SplashPanel;
 import net.sourceforge.jnlp.splashscreen.SplashUtils;
@@ -121,6 +118,8 @@ import sun.awt.X11.XEmbeddedFrame;
 import sun.misc.Ref;
 
 import com.sun.jndi.toolkit.url.UrlUtil;
+import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import net.sourceforge.jnlp.util.logging.OutputController;
 
 /*
  */
@@ -249,14 +248,17 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
         windowEventListener = new WindowAdapter() {
 
+            @Override
             public void windowClosing(WindowEvent evt) {
                 destroyApplet(identifier);
             }
 
+            @Override
             public void windowIconified(WindowEvent evt) {
                 appletStop();
             }
 
+            @Override
             public void windowDeiconified(WindowEvent evt) {
                 appletStart();
             }
@@ -267,11 +269,12 @@ public class PluginAppletViewer extends XEmbeddedFrame
         try {
             SwingUtilities.invokeAndWait(new SplashCreator(fPanel));
         } catch (Exception e) {
-            e.printStackTrace(); // Not much we can do other than  print
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e); // Not much we can do other than  print
         }
 
     }
 
+    @Override
     public void replaceSplash(final SplashPanel newSplash) {
         if (splashPanel == null) {
             return;
@@ -283,6 +286,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
         try {
             SwingUtilities.invokeAndWait(new Runnable() {
 
+                @Override
                 public void run() {
                     splashPanel.getSplashComponent().setVisible(false);
                     splashPanel.stopAnimation();
@@ -297,7 +301,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
                 }
             });
         } catch (Exception e) {
-            e.printStackTrace(); // Not much we can do other than print
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e); // Not much we can do other than print
         }
     }
 
@@ -309,6 +313,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
         try {
             SwingUtilities.invokeAndWait(new Runnable() {
 
+                @Override
                 public void run() {
                     splashPanel.getSplashComponent().setVisible(false);
                     splashPanel.stopAnimation();
@@ -324,7 +329,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
                 }
             });
         } catch (Exception e) {
-            e.printStackTrace(); // Not much we can do other than print
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e); // Not much we can do other than print
         }
     }
 
@@ -355,6 +360,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
             this.appletViewer = appletViewer;
         }
 
+        @Override
         public void appletStateChanged(AppletEvent evt) {
             AppletPanel src = (AppletPanel) evt.getSource();
 
@@ -386,10 +392,12 @@ public class PluginAppletViewer extends XEmbeddedFrame
                     // Swing also maintains a Frame list for the AppContext,
                     // so we will have to rearrange it as well.
                     //
-                    if (a != null)
+                    if (a != null) {
                         AppletPanel.changeFrameAppContext(frame, SunToolkit.targetToAppContext(a));
-                    else
+                    }
+                    else {
                         AppletPanel.changeFrameAppContext(frame, AppContext.getAppContext());
+                    }
 
                     updateStatus(appletViewer.identifier, PAV_INIT_STATUS.INIT_COMPLETE);
 
@@ -434,8 +442,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
         // If there is a key for this status, it means it
         // was either initialized before, or destroy has been
         // processed. Stop moving further.
-        if (updateStatus(identifier, PAV_INIT_STATUS.PRE_INIT) != null)
+        if (updateStatus(identifier, PAV_INIT_STATUS.PRE_INIT) != null) {
             return;
+        }
 
         // Extract the information from the message
         String[] msgParts = new String[4];
@@ -459,6 +468,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
                             "Height = ", height, "\n",
                             "DocumentBase = ", documentBase, "\n",
                             "Params = ", paramString);
+        JNLPRuntime.saveHistory(documentBase);
 
         PluginAppletPanelFactory factory = new PluginAppletPanelFactory();
         AppletMessageHandler amh = new AppletMessageHandler("appletviewer");
@@ -542,14 +552,15 @@ public class PluginAppletViewer extends XEmbeddedFrame
                     ;
 
                 // don't bother processing further for inactive applets
-                if (status.get(identifier).equals(PAV_INIT_STATUS.INACTIVE))
+                if (status.get(identifier).equals(PAV_INIT_STATUS.INACTIVE)) {
                     return;
+                }
 
                 applets.get(identifier).handleMessage(reference, message);
             }
         } catch (Exception e) {
 
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e);
 
             // If an exception happened during pre-init, we need to update status
             updateStatus(identifier, PAV_INIT_STATUS.INACTIVE);
@@ -639,6 +650,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
             PluginDebug.debug("Attempting to destroy panel ", identifier);
 
             SwingUtilities.invokeLater(new Runnable() {
+                @Override
                 public void run() {
                     pav.appletClose();
                 }
@@ -677,53 +689,63 @@ public class PluginAppletViewer extends XEmbeddedFrame
         PluginDebug.debug("Applet panel ", panel, " initialized");
     }
 
+    /* Resizes an applet panel, waiting for the applet to initialze. 
+     * Should be done asynchronously to avoid the chance of deadlock. */
+    private void resizeAppletPanel(final int width, final int height) {
+        // Wait for panel to come alive
+        waitForAppletInit(panel);
+
+        SwingUtilities.invokeLater(new Runnable() {
+            @Override
+            public void run() {
+                panel.updateSizeInAtts(height, width);
+
+                setSize(width, height);
+
+                // There is a rather odd drawing bug whereby resizing
+                // the panel makes no difference on initial call
+                // because the panel thinks that it is already the
+                // right size. Validation has no effect there either.
+                // So we work around by setting size to 1, validating,
+                // and then setting to the right size and validating
+                // again. This is not very efficient, and there is
+                // probably a better way -- but resizing happens
+                // quite infrequently, so for now this is how we do it
+
+                panel.setSize(1, 1);
+                panel.validate();
+
+                panel.setSize(width, height);
+                panel.validate();
+
+                panel.applet.resize(width, height);
+                panel.applet.validate();
+            }
+        });
+    }
+
     public void handleMessage(int reference, String message) {
         if (message.startsWith("width")) {
 
-            // Wait for panel to come alive
-            waitForAppletInit(panel);
-
             // 0 => width, 1=> width_value, 2 => height, 3=> height_value
             String[] dimMsg = message.split(" ");
 
-            final int height = Integer.parseInt(dimMsg[3]);
             final int width = Integer.parseInt(dimMsg[1]);
+            final int height = Integer.parseInt(dimMsg[3]);
 
-            panel.updateSizeInAtts(height, width);
+            /* Resize the applet asynchronously, to avoid the chance of 
+             * deadlock while waiting for the applet to initialize. 
+             * 
+             * In general, worker threads should spawn new threads for any blocking operations. */
+            Thread resizeAppletThread = new Thread("resizeAppletThread") {
+                @Override
+                public void run() {
+                    resizeAppletPanel(width, height);
+                }
+            };
 
-            try {
-                SwingUtilities.invokeAndWait(new Runnable() {
-                    public void run() {
-
-                        setSize(width, height);
-
-                        // There is a rather odd drawing bug whereby resizing
-                        // the panel makes no difference on initial call
-                        // because the panel thinks that it is already the
-                        // right size. Validation has no effect there either.
-                        // So we work around by setting size to 1, validating,
-                        // and then setting to the right size and validating
-                        // again. This is not very efficient, and there is
-                        // probably a better way -- but resizing happens
-                        // quite infrequently, so for now this is how we do it
-
-                        panel.setSize(1, 1);
-                        panel.validate();
-
-                        panel.setSize(width, height);
-                        panel.validate();
-
-                        panel.applet.resize(width, height);
-                        panel.applet.validate();
-                    }
-                });
-            } catch (InterruptedException e) {
-                // do nothing
-                e.printStackTrace();
-            } catch (InvocationTargetException e) {
-                // do nothing
-                e.printStackTrace();
-            }
+            /* Let it eventually complete */
+            resizeAppletThread.start();
 
         } else if (message.startsWith("GetJavaObject")) {
 
@@ -764,6 +786,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
     /**
      * Get an audio clip.
      */
+    @Override
     public AudioClip getAudioClip(URL url) {
         checkConnect(url);
         synchronized (audioClips) {
@@ -780,6 +803,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
     /**
      * Get an image.
      */
+    @Override
     public Image getImage(URL url) {
         return getCachedImage(url);
     }
@@ -808,8 +832,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
                 JNLPClassLoader loader = (JNLPClassLoader) panel.getAppletClassLoader();
 
                 URL localURL = null;
-                if (loader.resourceAvailableLocally(resourceName))
+                if (loader.resourceAvailableLocally(resourceName)) {
                     url = loader.getResource(resourceName);
+                }
 
                 url = localURL != null ? localURL : url;
             }
@@ -825,8 +850,8 @@ public class PluginAppletViewer extends XEmbeddedFrame
                 return ref;
             }
         } catch (Exception e) {
-            System.err.println("Error occurred when trying to fetch image:");
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Error occurred when trying to fetch image:");
+            OutputController.getLogger().log(e);
             return null;
         }
     }
@@ -843,6 +868,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
     /**
      * Get an applet by name.
      */
+    @Override
     public Applet getApplet(String name) {
         name = name.toLowerCase();
         SocketPermission panelSp =
@@ -873,6 +899,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
      * Return an enumeration of all the accessible
      * applets on this page.
      */
+    @Override
     public Enumeration<Applet> getApplets() {
         Vector<Applet> v = new Vector<Applet>();
         SocketPermission panelSp =
@@ -894,18 +921,21 @@ public class PluginAppletViewer extends XEmbeddedFrame
         return v.elements();
     }
 
-    /**
-     * Ignore.
-     */
+    @Override
     public void showDocument(URL url) {
         PluginDebug.debug("Showing document...");
         showDocument(url, "_self");
     }
 
-    /**
-     * Ignore.
-     */
+    @Override
     public void showDocument(URL url, String target) {
+        // If it is a javascript document, eval on current page.
+        if ("javascript".equals(url.getProtocol())) {
+            // Snip protocol off string
+            String evalString = url.toString().substring("javascript:".length());
+            eval(getWindow(), evalString);
+            return;
+        }
         try {
             Long reference = getRequestIdentifier();
             write("reference " + reference +  " LoadURL " + UrlUtil.encode(url.toString(), "UTF-8") + " " + target);
@@ -919,6 +949,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
     /**
      * Show status.
      */
+    @Override
     public void showStatus(String status) {
         try {
             // FIXME: change to postCallRequest
@@ -1113,8 +1144,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
             PluginDebug.debug("wait eval request 1");
             synchronized (request) {
                 PluginDebug.debug("wait eval request 2");
-                while (request.isDone() == false)
+                while (request.isDone() == false) {
                     request.wait();
+                }
                 PluginDebug.debug("wait eval request 3");
             }
         } catch (InterruptedException e) {
@@ -1178,8 +1210,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
             PluginDebug.debug("wait call request 1");
             synchronized (request) {
                 PluginDebug.debug("wait call request 2");
-                while (request.isDone() == false)
+                while (request.isDone() == false) {
                     request.wait();
+                }
                 PluginDebug.debug("wait call request 3");
             }
         } catch (InterruptedException e) {
@@ -1202,7 +1235,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
                             " " + encodedURI, reference);
 
         } catch (UnsupportedEncodingException e) {
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e);
             return null;
         }
 
@@ -1213,8 +1246,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
             PluginDebug.debug("wait cookieinfo request 1");
             synchronized (request) {
                 PluginDebug.debug("wait cookieinfo request 2");
-                while (request.isDone() == false)
+                while (request.isDone() == false) {
                     request.wait();
+                }
                 PluginDebug.debug("wait cookieinfo request 3");
             }
         } catch (InterruptedException e) {
@@ -1225,32 +1259,18 @@ public class PluginAppletViewer extends XEmbeddedFrame
         return request.getObject();
     }
 
-    public static Object requestPluginProxyInfo(URI uri) {
-
-        String requestURI = null;
+    /**
+     * Obtain information about the proxy from the browser.
+     *
+     * @param uri a String in url-encoded form
+     * @return a {@link URI} that indicates a proxy.
+     */
+    public static Object requestPluginProxyInfo(String uri) {
         Long reference = getRequestIdentifier();
 
-        try {
-
-            // there is no easy way to get SOCKS proxy info. So, we tell mozilla that we want proxy for
-            // an HTTP uri in case of non http/ftp protocols. If we get back a SOCKS proxy, we can
-            // use that, if we get back an http proxy, we fallback to DIRECT connect
-
-            String scheme = uri.getScheme();
-            String port = uri.getPort() != -1 ? ":" + uri.getPort() : "";
-            if (!uri.getScheme().startsWith("http") && !uri.getScheme().equals("ftp"))
-                scheme = "http";
-
-            requestURI = UrlUtil.encode(scheme + "://" + uri.getHost() + port + "/" + uri.getPath(), "UTF-8");
-        } catch (Exception e) {
-            PluginDebug.debug("Cannot construct URL from ", uri.toString(), " ... falling back to DIRECT proxy");
-            e.printStackTrace();
-            return null;
-        }
-
         PluginCallRequest request = requestFactory.getPluginCallRequest("proxyinfo",
                 "plugin PluginProxyInfo reference " + reference + " " +
-                        requestURI, reference);
+                        uri, reference);
 
         PluginMessageConsumer.registerPriorityWait(reference);
         streamhandler.postCallRequest(request);
@@ -1259,8 +1279,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
             PluginDebug.debug("wait call request 1");
             synchronized (request) {
                 PluginDebug.debug("wait call request 2");
-                while (request.isDone() == false)
+                while (request.isDone() == false) {
                     request.wait();
+                }
                 PluginDebug.debug("wait call request 3");
             }
         } catch (InterruptedException e) {
@@ -1285,8 +1306,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
             PluginDebug.debug("wait finalize request 1");
             synchronized (request) {
                 PluginDebug.debug("wait finalize request 2");
-                while (request.isDone() == false)
+                while (request.isDone() == false) {
                     request.wait();
+                }
                 PluginDebug.debug("wait finalize request 3");
             }
         } catch (InterruptedException e) {
@@ -1325,6 +1347,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
         PluginDebug.debug("WRITING 2 DONE");
     }
 
+    @Override
     public void setStream(String key, InputStream stream) throws IOException {
         // We do nothing.
     }
@@ -1405,6 +1428,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
         }
 
         AccessController.doPrivileged(new PrivilegedAction<Void>() {
+            @Override
             public Void run() {
                 ((AppletViewerPanel)panel).createAppletThread();
                 return null;
@@ -1416,6 +1440,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
         panel.sendEvent(AppletPanel.APPLET_START);
     }
 
+    @Override
     public int print(Graphics graphics, PageFormat pf, int pageIndex) {
         return Printable.NO_SUCH_PAGE;
     }
@@ -1461,13 +1486,15 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
         new Thread(new Runnable() {
             @SuppressWarnings("deprecation")
+            @Override
             public void run() {
                 ClassLoader cl = p.applet.getClass().getClassLoader();
 
                 // Since we want to deal with JNLPClassLoader, extract it if this
                 // is a codebase loader
-                if (cl instanceof JNLPClassLoader.CodeBaseClassLoader)
+                if (cl instanceof JNLPClassLoader.CodeBaseClassLoader) {
                     cl = ((JNLPClassLoader.CodeBaseClassLoader) cl).getParentJNLPClassLoader();
+                }
 
                 appletShutdown(p);
                 appletPanels.removeElement(p);
@@ -1477,6 +1504,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
                 try {
                     SwingUtilities.invokeAndWait(new Runnable() {
+                        @Override
                         public void run() {
                             dispose();
                         }
@@ -1517,10 +1545,12 @@ public class PluginAppletViewer extends XEmbeddedFrame
             try {
                 java.security.Permission perm =
                         url.openConnection().getPermission();
-                if (perm != null)
+                if (perm != null) {
                     security.checkPermission(perm);
-                else
+                }
+                else {
                     security.checkConnect(url.getHost(), url.getPort());
+                }
             } catch (java.io.IOException ioe) {
                 security.checkConnect(url.getHost(), url.getPort());
             }
@@ -1534,6 +1564,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
      * the parent class's update() just does a couple of checks (both of
      * which are accounted for) and then calls paint anyway.
      */
+    @Override
     public void paint(Graphics g) {
 
         // If the image or the graphics don't exist, create new ones
@@ -1576,8 +1607,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
                                        long timeout) {
 
         // Can't wait on null. Return 0 indicating no wait happened.
-        if (lock == null)
-            return 0;
+        if (lock == null) {
+                                               return 0;
+                                           }
 
         assert lock.isHeldByCurrentThread();
 
@@ -1601,6 +1633,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
             this.fPanel = fPanel;
         }
 
+        @Override
         public void run() {
             add("Center", fPanel);
             fPanel.setVisible(false);
