@@ -37,13 +37,19 @@ exception statement from your version. */
 
 package sun.applet;
 
+import java.io.UnsupportedEncodingException;
 import java.net.InetSocketAddress;
 import java.net.Proxy;
 import java.net.URI;
+import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.List;
 
+import com.sun.jndi.toolkit.url.UrlUtil;
+
+import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.JNLPProxySelector;
+import net.sourceforge.jnlp.util.logging.OutputController;
 import net.sourceforge.jnlp.util.TimedHashMap;
 
 /**
@@ -59,6 +65,10 @@ public class PluginProxySelector extends JNLPProxySelector {
 
     private TimedHashMap<String, Proxy> proxyCache = new TimedHashMap<String, Proxy>();
 
+    public PluginProxySelector(DeploymentConfiguration config) {
+        super(config);
+    }
+
     /**
      * Selects the appropriate proxy (or DIRECT connection method) for the given URI
      *
@@ -78,8 +88,19 @@ public class PluginProxySelector extends JNLPProxySelector {
         }
 
         // Nothing usable in cache. Fetch info from browser
+
+        String requestURI;
+        try {
+            requestURI = convertUriSchemeForProxyQuery(uri);
+        } catch (Exception e) {
+            PluginDebug.debug("Cannot construct URL from ", uri.toString(), " ... falling back to DIRECT proxy");
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e);
+            proxyList.add(Proxy.NO_PROXY);
+            return proxyList;
+        }
+
         Proxy proxy = Proxy.NO_PROXY;
-        Object o = PluginAppletViewer.requestPluginProxyInfo(uri);
+        Object o = getProxyFromRemoteCallToBrowser(requestURI);
 
         // If the browser returned anything, try to parse it. If anything in the try block fails, the fallback is direct connection
         try {
@@ -95,14 +116,14 @@ public class PluginProxySelector extends JNLPProxySelector {
 
                     proxy = new Proxy(type, socketAddr);
 
-                    String uriKey = uri.getScheme() + "://" + uri.getHost();
+                    String uriKey = computeKey(uri);
                     proxyCache.put(uriKey, proxy);
                 } else {
                     PluginDebug.debug("Proxy ", proxyURI, " cannot be used for ", uri, ". Falling back to DIRECT");
                 }
             }
         } catch (Exception e) {
-            e.printStackTrace();
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL,e);
         }
 
         proxyList.add(proxy);
@@ -112,6 +133,11 @@ public class PluginProxySelector extends JNLPProxySelector {
         return proxyList;
     }
 
+    /** For tests to override */
+    protected Object getProxyFromRemoteCallToBrowser(String uri) {
+        return PluginAppletViewer.requestPluginProxyInfo(uri);
+    }
+
     /**
      * Checks to see if proxy information is already cached.
      *
@@ -119,8 +145,7 @@ public class PluginProxySelector extends JNLPProxySelector {
      * @return The cached Proxy. null if there is no suitable cached proxy.
      */
     private Proxy checkCache(URI uri) {
-
-        String uriKey = uri.getScheme() + "://" + uri.getHost();
+        String uriKey = computeKey(uri);
         if (proxyCache.get(uriKey) != null) {
             return proxyCache.get(uriKey);
         }
@@ -128,4 +153,23 @@ public class PluginProxySelector extends JNLPProxySelector {
         return null;
     }
 
+    /** Compute a key to use for the proxy cache */
+    private String computeKey(URI uri) {
+        return uri.getScheme() + "://" + uri.getHost();
+    }
+
+    public static String convertUriSchemeForProxyQuery(URI uri) throws URISyntaxException, UnsupportedEncodingException {
+        // there is no easy way to get SOCKS proxy info. So, we tell mozilla that we want proxy for
+        // an HTTP uri in case of non http/ftp protocols. If we get back a SOCKS proxy, we can
+        // use that, if we get back an http proxy, we fallback to DIRECT connect
+
+        String scheme = uri.getScheme();
+        if (!scheme.startsWith("http") && !scheme.equals("ftp")) {
+            scheme = "http";
+        }
+
+        URI result = new URI(scheme, uri.getUserInfo(), uri.getHost(), uri.getPort(),
+                uri.getPath(), uri.getQuery(), uri.getFragment());
+        return UrlUtil.encode(result.toString(), "UTF-8");
+    }
 }
