@@ -58,15 +58,16 @@ public class ParserTest {
     private static final Locale LANG_COUNTRY_LOCALE = new Locale(LANG, COUNTRY);
     private static final Locale ALL_LOCALE = new Locale(LANG, COUNTRY, VARIANT);
 
+    ParserSettings defaultParser=new ParserSettings();
     @Test(expected = MissingInformationException.class)
     public void testMissingInfoFullLocale() throws ParseException {
         String data = "<jnlp></jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         parser.getInfo(root);
     }
 
@@ -77,11 +78,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -100,11 +101,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
 
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
@@ -130,10 +131,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly two info descs should be found", infoDescs.size() == 2);
 
@@ -157,10 +158,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly two info descs should be found", infoDescs.size() == 2);
 
@@ -180,10 +181,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
 
@@ -211,10 +212,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -242,10 +243,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -272,10 +273,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -301,10 +302,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -329,10 +330,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -356,10 +357,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -380,10 +381,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -404,10 +405,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -426,11 +427,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -449,11 +450,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -488,11 +489,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(ALL_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
 
         Assert.assertTrue("Exactly five info descs should be found", infoDescs.size() == 5);
@@ -511,11 +512,11 @@ public class ParserTest {
     public void testMissingInfoLangCountryLocale() throws ParseException {
         String data = "<jnlp></jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         parser.getInfo(root);
     }
 
@@ -526,11 +527,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -549,11 +550,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
 
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
@@ -575,10 +576,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly two info descs should be found", infoDescs.size() == 2);
 
@@ -602,10 +603,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly two info descs should be found", infoDescs.size() == 2);
 
@@ -625,10 +626,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
 
@@ -656,10 +657,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -687,10 +688,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -717,10 +718,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -746,10 +747,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -774,10 +775,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -801,10 +802,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -825,10 +826,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -849,10 +850,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -871,11 +872,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -894,11 +895,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -933,11 +934,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_COUNTRY_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
 
         Assert.assertTrue("Exactly five info descs should be found", infoDescs.size() == 5);
@@ -956,11 +957,11 @@ public class ParserTest {
     public void testMissingInfoLangLocale() throws ParseException {
         String data = "<jnlp></jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         parser.getInfo(root);
     }
 
@@ -971,11 +972,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -994,11 +995,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
 
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
@@ -1016,10 +1017,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
 
@@ -1040,10 +1041,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly two info descs should be found", infoDescs.size() == 2);
 
@@ -1063,10 +1064,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly one info desc should be found", infoDescs.size() == 1);
 
@@ -1094,10 +1095,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -1125,10 +1126,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly three info descs should be found", infoDescs.size() == 3);
 
@@ -1151,10 +1152,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
         Assert.assertTrue("Exactly two info descs should be found", infoDescs.size() == 2);
 
@@ -1179,10 +1180,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -1206,10 +1207,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -1230,10 +1231,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -1254,10 +1255,10 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -1276,11 +1277,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -1299,11 +1300,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = new ArrayList<InformationDesc>();
         infoDescs.addAll(parser.getInfo(root));
 
@@ -1338,11 +1339,11 @@ public class ParserTest {
                 + "  </information>\n"
                 + "</jnlp>\n";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false);
+        Parser parser = new Parser(file, null, root, defaultParser);
         List<InformationDesc> infoDescs = parser.getInfo(root);
 
         Assert.assertTrue("Exactly five info descs should be found", infoDescs.size() == 5);
@@ -1366,12 +1367,12 @@ public class ParserTest {
                 ">\n" +
                 "</jnlp>";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         URL overwrittenCodebase = new URL("http://icedtea.classpath.org");
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false, overwrittenCodebase);
+        Parser parser = new Parser(file, null, root, defaultParser, overwrittenCodebase);
 
         Assert.assertEquals("http://www.redhat.com/", parser.getCodeBase().toExternalForm());
     }
@@ -1385,12 +1386,12 @@ public class ParserTest {
                 ">\n" +
                 "</jnlp>";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         URL overwrittenCodebase = new URL("http://icedtea.classpath.org");
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false, overwrittenCodebase);
+        Parser parser = new Parser(file, null, root, defaultParser, overwrittenCodebase);
 
         Assert.assertEquals(overwrittenCodebase.toExternalForm(), parser.getCodeBase().toExternalForm());
     }
@@ -1403,12 +1404,12 @@ public class ParserTest {
                 ">\n" +
                 "</jnlp>";
 
-        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()));
+        Node root = Parser.getRootNode(new ByteArrayInputStream(data.getBytes()), defaultParser);
         Assert.assertEquals("Root name is not jnlp", "jnlp", root.getNodeName());
         URL overwrittenCodebase = new URL("http://icedtea.classpath.org");
 
         MockJNLPFile file = new MockJNLPFile(LANG_LOCALE);
-        Parser parser = new Parser(file, null, root, false, false, overwrittenCodebase);
+        Parser parser = new Parser(file, null, root, defaultParser, overwrittenCodebase);
 
         Assert.assertEquals(overwrittenCodebase.toExternalForm(), parser.getCodeBase().toExternalForm());
     }
